# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net
# Modified by dgranda

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import logging
import os
import traceback
from lxml import etree
from pytrainer.lib.date import getDateTime

class gpxplus():
	def __init__(self, parent = None, data_path = None):
		self.parent = parent
		self.pytrainer_main = parent.parent
		self.tmpdir = self.pytrainer_main.profile.tmpdir
		self.main_data_path = data_path
		self.data_path = os.path.dirname(__file__)
		self.xmldoc = None
		self.activitiesSummary = []

	def getXmldoc(self):
		''' Function to return parsed xmlfile '''
		return self.xmldoc

	def getFileType(self):
		return _("GPS eXchange file")

	def getActivitiesSummary(self):
		return self.activitiesSummary

	def testFile(self, filename):
		logging.debug('>>')
		logging.debug("Testing " + filename)
		#Check if file is a GPX
		try:
			#parse as xml
			xmldoc = etree.parse(filename)
			#Parse XML schema
			xmlschema_doc = etree.parse(self.main_data_path+"schemas/Topografix_gpx11.xsd")
			xmlschema = etree.XMLSchema(xmlschema_doc)
			if (xmlschema.validate(xmldoc)):
				#Valid gpx file
				self.xmldoc = xmldoc
				startTime = getDateTime(self.startTimeFromFile(xmldoc))
				indatabase = self.inDatabase(xmldoc, startTime)
				sport = self.getSport(xmldoc)
				duration  = self.getDetails(xmldoc, startTime)
				distance = ""
				self.activitiesSummary.append( (0,
												indatabase,
												startTime[1].strftime("%Y-%m-%dT%H:%M:%S"),
												distance ,
												str(duration),
												sport,
												) )
				return True
		except:
			#Not gpx file
			logging.debug("Traceback: %s" % traceback.format_exc())
			return False
		return False

	def inDatabase(self, tree, startTime):
		#comparing date and start time (sport may have been changed in DB after import)
		time = startTime
		if time is None:
			return False
		time = time[0].strftime("%Y-%m-%dT%H:%M:%SZ")
		if self.parent.parent.ddbb.select("records","*","date_time_utc=\"%s\"" % (time)):
			return True
		else:
			return False

	def getDetails(self, tree, startTime):
		root = tree.getroot()
		#Get all times from file
		times = root.findall(".//{http://www.topografix.com/GPX/1/1}time")
		time = times[-1].text
		return getDateTime(time)[0]-startTime[0]

	def getSport(self, tree):
		#No sport in GPX file
		return None

	def startTimeFromFile(self, tree):
		""" Function to return the first time element from a GPX 1.1 file (skipping not mandatory metadata section) """
		root = tree.getroot()
		timeElement = root.xpath(".//g:time[not(parent::g:metadata)]", namespaces={'g':'http://www.topografix.com/GPX/1/1'})[0]
		if timeElement is not None:
			return timeElement.text
		return None

	def getGPXFile(self, ID, file_id):
		"""
			Generate GPX file based on activity ID

			Returns (sport, GPX filename)
		"""
		sport = None
		gpxFile = None
		if ID == "0": #Only one activity in file
			gpxFile = "%s/gpx-%s-%s.gpx" % (self.tmpdir, file_id, ID)
			sport = self.getSport(self.xmldoc)
			self.createGPXfile(gpxFile, self.xmldoc)
		return sport, gpxFile

	def createGPXfile(self, gpxfile, tree):
		tree.write(gpxfile, xml_declaration=True, encoding='UTF-8')




"""Encode/decode Google Maps points encoding
Matt King, 2005-06-16
"""

import random

def decodePoints(points):
    """Decodes a string of locations encoded using the GMap encoding
    Accepts the encoded string and
    returns an array of lat/lon pairs [lat1,lon1,lat2,lon2,...]"""
    if not points:
        return []
    Ch = len(points)
    pb = 0
    locations = []
    Ka = 0
    Pa = 0
    while pb < Ch:
        oc = 0
        Fa = 0
        while 1:
            ub = ord(points[pb])-63
            pb += 1
            Fa |= (ub&31)<<oc
            oc += 5
            if ub < 32: break
        if Fa & 1: i = ~(Fa>>1)
        else: i = Fa>>1
        Ka = Ka+i
        locations.append(Ka*1.0E-5)

        oc = 0
        Fa = 0
        while 1:
            ub = ord(points[pb])-63
            pb += 1
            Fa |= (ub&31)<<oc
            oc += 5
            if ub < 32: break
        if Fa & 1: i = ~(Fa>>1)
        else: i = Fa>>1
        Pa = Pa+i

        locations.append(Pa*1.0E-5)

    return locations


def encodePoints(locations):
    """Encodes lat/lon locations into a Gmap polyline encoding.
    Accepts an array of lat/lon pairs [lat1,lon1,lat2,lon2,...] and
    returns 2 strings, the encoded points and the corresponding levels"""
    points = []
    levels = []
    xo = yo = 0
    for i in locations:
        y = int(float(i[0])*100000)
        yd = y - yo
        yo = y
        f = (abs(yd) << 1) - (yd<0)
        while 1:
            e = f & 31
            f >>= 5
            if f:
                e |= 32
            points.append(chr(e+63))
            if f == 0: break

        x = int(float(i[1])*100000)
        xd = x - xo
        xo = x
        f = (abs(xd) << 1) - (xd<0)
        while 1:
            e = f & 31
            f >>= 5
            if f:
                e |= 32
            points.append(chr(e+63))
            if f == 0: break

        levels.append(nextLevel())
    levels[0] = 'B'
    levels[-1] = 'B'
    return "".join(points), "".join(levels)

def nextLevel():
    r = random.random()
    if r < 0.65: return '?'
    if r < 0.92: return '@'
    if r < 0.97: return 'A'
    return 'B'

if __name__ == '__main__':
    locs = [(38.5, -120.2), (40.7, -120.95), (43.252, -126.453)]
    locs = [(37.4419, -122.1419),(37.4519, -122.1519),( 37.4619, -122.1819)]
    points, levels = encodePoints(locs)
    print points
    print levels
    decodedLocs = decodePoints(points)
    print decodePoints(points)
    def _cmpFloat(x,y):
        if abs(x-y) > 0.0001: return 0
        return 1

    assert(len(locs) == len(decodedLocs))
    assert(len(levels) == 3)
    for i in range(len(locs)):
        assert(_cmpFloat(locs[i], decodedLocs[i]))

    



# Open Street Maps

# TODO: Add Google satellite images layers ?
# TODO: Create map class/interface that osm/googlemaps will inherit from ?

import os
import re
import logging
import urllib           # for downloading cached versions of openlayers.js and openstreetmaps.js
import time             # Used for checking if local cached file is current
    
from pytrainer.lib.gpx import Gpx
import pytrainer.lib.points as Points
from pytrainer.lib.fileUtils import fileUtils
from pytrainer.record import Record
from pytrainer.lib.uc import UC

class Osm:
    # Default URLS
    URLS = {'OpenLayers.js' : "http://openlayers.org/api/OpenLayers.js",
            'OpenStreetMap.js' : "http://www.openstreetmap.org/openlayers/OpenStreetMap.js"}
    # URLS that will not be cached
    staticURLS = {'OpenLayers' : 'http://openlayers.org/api/'}                 #openlayers default location

    def __init__(self, data_path = None, waypoint = None, pytrainer_main=None):
        logging.debug(">>")
        self.data_path = data_path
        self.waypoint = waypoint
        self.pytrainer_main = pytrainer_main
        self.tmpdir = (self.pytrainer_main.profile.tmpdir)
        self.htmlfile = "%s/osm.html" % (self.tmpdir)
        self.uc = UC()
        logging.debug("<<")

    def download(self,url,localfile):
        """Copy the contents of a file from a given URL to pytrainer's tmpdir
        """    
        logging.debug(">>")
        print "Downloading %s" % (url)
        webFile = urllib.urlopen(url)
        # always store downloaded files in tmpdir/cache
        localFile = open(self.tmpdir + '/cache/' + localfile, 'w')
        localFile.write(webFile.read())
        webFile.close()
        localFile.close()
        logging.debug("<<")
        
    def cacheUrls(self):
        ''' Store URL copies of needed files locally, 
            download new versions every ~14 days or if files does'nt exists
        '''
        #TODO: open a msg/progress bar while downloading the new files to inform the user
        logging.debug(">>")
        # create cache subfolder in tmpdir if not already there
        try:
            cachedir = self.tmpdir + '/cache';
            if not os.path.isdir(cachedir):
                print "Creating %s folder" % (cachedir)
                os.mkdir(cachedir)
                
            for localfile in self.URLS:
                # local cached file does not exists? download it
                if not os.path.isfile(cachedir + '/' + localfile):
                    self.download(self.URLS[localfile],localfile)
                else: 
                    creationTime = os.path.getctime(cachedir + '/' + localfile)     # read file creation time
                    if creationTime > time.time():                                  # file is in the future?! download it again
                        self.download(self.URLS[localfile],localfile)
                    elif creationTime + 14*24*60*60 < time.time():                  # 14 (days) * 24 hours * 60 minutes * 60 seconds = 14 days 
                        self.download(self.URLS[localfile],localfile)           
                # No exception was thrown, assuming local cache file exists and current
                self.URLS[localfile]='file://' + cachedir + '/' + localfile;
                logging.info("Using %s file " % (self.URLS[localfile]))
        except Exception as e:
            loggin.error("(%s) Error while downloading %s to local cache, using default hosted file instaed." \
                           % (str(e), self.URLS[localfile]))
        logging.debug("<<")
    
    def drawMap(self, activity, linetype):
        '''Draw OSM map
        create HTML file using Open Layers and Open Street Map
        render using embedded Webkit
        '''
        logging.debug(">>")
        points = []
        levels = []
        pointlist = []
        polyline = []
        attrlist = []

        try :
            list_values = activity.tracks
            if list_values is not None and list_values != [] and len(list_values) > 0:
                for i in list_values:
                    lat, lon = float(i[4]), float(i[5])
                    pointlist.append((lat,lon))
                    polyline.append("[%s, %s]" % (lon, lat))
                    attrlist.append((i[3],i[6])) # (Speed, HR)
                points,levels = Points.encodePoints(pointlist)
                points = points.replace("\\","\\\\")
                laps = activity.laps
                timeHours = int(activity.time) / 3600
                timeMin = (float(activity.time) / 3600.0 - timeHours) * 60
                time = "%d%s %02d%s" % (timeHours, _("h"), timeMin, _("min"))
                startinfo = "<div class='info_content'>%s: %s</div>" % (activity.sport_name, activity.title)
                finishinfo = "<div class='info_content'>%s: %s<br>%s: %s%s</div>" % (_("Time"), \
                            time, _("Distance"), activity.distance, self.uc.unit_distance)
                startinfo = startinfo.encode('ascii', 'xmlcharrefreplace') #Encode for html
                finishinfo = finishinfo.encode('ascii', 'xmlcharrefreplace') #Encode for html

                self.createHtml_osm(polyline, startinfo, finishinfo, laps, attrlist, linetype)
            else:
                self.createErrorHtml()
        except Exception as e:
            self.createErrorHtml(e)
        
        return self.htmlfile
        logging.debug("<<")

    def selectArea(self,dc=None):
    
        # try using local cached versions of JS files for faster rendering
        self.cacheUrls();

        content = '''
        <html xmlns="http://www.w3.org/1999/xhtml">
        <head>
        <link rel="stylesheet" href="../theme/default/style.css" type="text/css" />
        <link rel="stylesheet" href="style.css" type="text/css" />
        <style type="text/css">
            p {
                width: 512px;
            }
            #config {
                margin-top: 1em;
                width: 512px;
                position: relative;
                height: 8em;
            }
            #controls {
                padding-left: 2em;
                margin-left: 0;
                width: 12em;
            }
            #controls li {
                padding-top: 0.5em;
                list-style: none;
            }
            #options {
                font-size: 1em;
                top: 0;
                margin-left: 15em;
                position: absolute;
            }

            /* avoid pink tiles */
            .olImageLoadError {
                background-color: transparent !important;
            }
        </style>

        <!-- Load libraries from remote or local cache -->
        <script src="''' + self.URLS['OpenLayers.js']    + '''"></script>
        <script src="''' + self.URLS['OpenStreetMap.js'] + '''"></script>

        <script type="text/javascript">
            var map, polygonControl;
            function init(){

                //"fix" locations
                OpenLayers.ImgPath="''' + self.staticURLS['OpenLayers'] + '''img/";
                OpenLayers.scriptLocation="''' + self.staticURLS['OpenLayers'] + '''";
                OpenLayers._getScriptLocation=function() { return "''' + self.staticURLS['OpenLayers'] + '''";};
                            
                // for transforming WGS 1984 to Spherical Mercator Projection
                pWGS = new OpenLayers.Projection("EPSG:4326");
                pMP = new OpenLayers.Projection("EPSG:900913");

                map = new OpenLayers.Map ("map", {
                    controls:[
                        new OpenLayers.Control.Navigation(),
                        new OpenLayers.Control.PanZoomBar(),
                        new OpenLayers.Control.LayerSwitcher(),
                        new OpenLayers.Control.Attribution(),
                        new OpenLayers.Control.MousePosition()],
                    maxExtent: new OpenLayers.Bounds(-20037508.34,-20037508.34,20037508.34,20037508.34),
                    maxResolution: 156543.0399,
                    numZoomLevels: 19,
                    units: 'm',
                    projection: pMP,
                    displayProjection: pWGS
                } );

                //Add open street maps layers
                layerMapnik = new OpenLayers.Layer.OSM.Mapnik("Mapnik");
                map.addLayer(layerMapnik);

                //Add polygon drawing layer
                var polygonLayer = new OpenLayers.Layer.Vector("Polygon Layer");
                map.addLayer(polygonLayer);
                
                polyOptions = {sides: 4, irregular: true};
                polygonControl = new OpenLayers.Control.DrawFeature(polygonLayer,
                                                OpenLayers.Handler.RegularPolygon,
                                                {handlerOptions: polyOptions});

                // For desrializings
                var out_options = {
                    'internalProjection': map.baseLayer.projection,
                    'externalProjection': pWGS
                };

                // allow only one feature to be drawn
                polygonLayer.events.register("beforefeatureadded", polygonLayer, function(f) { polygonLayer.removeAllFeatures(true)});
                // Change title after draw
                polygonLayer.events.register("featureadded", polygonLayer, function(f) { 
                    var title=(new OpenLayers.Format.GeoJSON(out_options).write(f.feature,false));
                    document.title=title;
                    });
                
                //prepaint saved polygon
                savedPolygon = \'''' + str(self.waypoint) + '''\';
                if ((savedPolygon) && (savedPolygon!=='None')) {            
                   // deserialize 
                   var geojson_format = new OpenLayers.Format.GeoJSON(out_options);
                   var feature = geojson_format.read(savedPolygon);
                   // make sure it's an array
                   if(feature.constructor != Array)
                        feature = [feature];
                   if (feature && feature.length>0) {
                      var bounds = feature[0].geometry.getBounds();
                      polygonLayer.addFeatures(feature);
                      map.zoomToExtent(bounds);
                   }
                } else {
                    map.setCenter(new OpenLayers.LonLat(0, 0), 3);
                }         

                map.addControl(polygonControl);            
                
                document.getElementById('noneToggle').checked = true;
                
            }
            
            function setOptions(options) {
                polygonControl.handler.setOptions(options);
            }
            
            function setSize(fraction) {
                var radius = fraction * map.getExtent().getHeight();
                polygonControl.handler.setOptions({radius: radius,
                                                   angle: 0});
            }
        </script>
        </head>

          <body onload="init()">
            <input type="radio" name="type"
                   value="none" id="noneToggle"
                   onclick="polygonControl.deactivate()"
                   checked="checked" />
            <label for="noneToggle">Navigate</label>
            <input type="radio" name="type"
                   value="polygon" id="polygonToggle"
                   onclick="polygonControl.activate()" />
            <label for="polygonToggle">Draw polygon</label>
            <div id="map" class="smallmap"></div>   
          </body>
        </html>'''     

        file = fileUtils(self.htmlfile,content)
        file.run()
        return self.htmlfile
        
    def createHtml_osm(self, polyline, startinfo, finishinfo, laps, attrlist, linetype):
        '''
        Generate OSM map html file using MapLayers
        '''
        logging.debug(">>")

        # try using local cached versions of JS files for faster rendering
        self.cacheUrls();
        
        content = '''<html>
        <head>
            <!-- bring in the OpenLayers javascript library
                 (here we bring it from the remote site, but you could
                 easily serve up this javascript yourself) -->
            <script src="''' + self.URLS['OpenLayers.js'] + '''"></script>
            <!-- bring in the OpenStreetMap OpenLayers layers.
                 Using this hosted file will make sure we are kept up
                 to date with any necessary changes -->
            <script src="''' + self.URLS['OpenStreetMap.js'] + '''"></script>

            <script type="text/javascript">
                //complex object of type OpenLayers.Map
                var map;

                //icons data object
                var icons = {
                    iconSize : new OpenLayers.Size(30,30)'''

        # If have laps data insert markers here
        try:
            lapsContent=''
            for lap in laps[:500]:  # OpenLayers with firefox is limited to 500 markers -> TODO: Transfer to a constant somewhere ?
                lapNumber = int(lap['lap_number'])+1
                elapsedTime = float(lap['elapsed_time'])
                elapsedTimeHours = int(elapsedTime/3600)
                elapsedTimeMins = int((elapsedTime - (elapsedTimeHours * 3600)) / 60)
                elapsedTimeSecs = elapsedTime - (elapsedTimeHours * 3600) - (elapsedTimeMins * 60)
                if elapsedTimeHours > 0:
                    strElapsedTime = "%0.0dh:%0.2dm:%0.2fs" % (elapsedTimeHours, elapsedTimeMins, elapsedTimeSecs)
                elif elapsedTimeMins > 0:
                    strElapsedTime = "%0.0dm:%0.2fs" % (elapsedTimeMins, elapsedTimeSecs)
                else:
                    strElapsedTime = "%0.0fs" % (elapsedTimeSecs)
                #process lat and lon for this lap
                lapLat = float(lap['end_lat'])
                lapLon = float(lap['end_lon'])
                #build laps content string
                lapsContent+=',\n'
                lapsContent+='\t\t\t\t\tlap%d: { url : "/waypoint.png", coordinates : [%f,%f], popupInfo: "%s" }' % \
                        (lapNumber, lapLon, lapLat, \
                        "<div class='info_content'>End of lap:%d<br>Elapsed time:%s<br>Distance:%0.2f km<br>Calories:%s</div>" % \
                            (lapNumber, strElapsedTime, float(lap['distance'])/1000, lap['calories'])
                        )
            content+=lapsContent
        except Exception as e:
            # If something breaks here just skip laps data
            logging.error('Error formating laps data: ' + str(e))
        # Insert start/finish track markers
        content+=''',\n        start : { url : "/start.png", coordinates : %s, popupInfo : "%s" },
                    finish : { url : "/finish.png", coordinates : %s, popupInfo : "%s" },
                    url : "file://%s/glade"''' \
                    % (polyline[0], startinfo, polyline[-1], finishinfo, os.path.abspath(self.data_path))

        content+='''};\n
                function init() {

                // fool openlayers scripts so it will download images and themes from the web instead of local folder if cached
                OpenLayers.ImgPath="''' + self.staticURLS['OpenLayers'] + '''img/";
                OpenLayers.scriptLocation="''' + self.staticURLS['OpenLayers'] + '''";
                OpenLayers._getScriptLocation=function() { return "''' + self.staticURLS['OpenLayers'] + '''";};

                // for transforming WGS 1984 to Spherical Mercator Projection
                pWGS = new OpenLayers.Projection("EPSG:4326");
                pMP = new OpenLayers.Projection("EPSG:900913");

                map = new OpenLayers.Map ("map", {
                    controls:[
                        new OpenLayers.Control.Navigation(),
                        new OpenLayers.Control.PanZoomBar(),
                        new OpenLayers.Control.LayerSwitcher(),
                        new OpenLayers.Control.Attribution()],
                    maxExtent: new OpenLayers.Bounds(-20037508.34,-20037508.34,20037508.34,20037508.34),
                    maxResolution: 156543.0399,
                    numZoomLevels: 19,
                    units: 'm',
                    projection: pMP,
                    displayProjection: pWGS
                } );

                // Track painting style
                var trackStyle = {
                    strokeColor: "#33DDDD",
                    strokeWidth: 3,
                    strokeDashstyle: "solid",
                    strokeOpacity: 0.6,
                    pointRadius: 6,
                };

                //Build track object
                var track =
                    {
                    "type":"Feature",
                    "id":"OpenLayers.Feature.Vector_259",
                    "properties":{},
                    "geometry":
                    {
                        "type":"LineString",
                        "coordinates":
                            ['''
        #Insert track points here
        content+=",".join(polyline);
        content+=''']
                    },
                    "crs":
                        {
                        "type":"OGC",
                        "properties":
                            {
                            "urn":"urn:ogc:def:crs:OGC:1.3:CRS84"
                            }
                        }
                    }

                //Add open street maps layers
                layerMapnik = new OpenLayers.Layer.OSM.Mapnik("Mapnik");
                map.addLayer(layerMapnik);

                //Create vector layer to add the data on to
                var vector_layer = new OpenLayers.Layer.Vector();
                vector_layer.setName('Track');

                var geojson_format = new OpenLayers.Format.GeoJSON();
                var feature = geojson_format.read(track,"Feature");

                // transform from WGS 1984 to Spherical Mercator Projection
                feature.geometry.transform(pWGS, pMP);

                feature.geometry.calculateBounds();
                var vector=new OpenLayers.Feature.Vector();
                vector.geometry = feature.geometry;
                vector.style=trackStyle;

                vector_layer.addFeatures(vector);
                map.addLayer(vector_layer);

                // Insert start/finish markers
                layerMarkers = new OpenLayers.Layer.Markers("Markers");
                var offset = new OpenLayers.Pixel(-(icons.iconSize.w/2), -icons.iconSize.h);
                for (var i in icons) {
                    if (icons[i].coordinates) {
                        icons[i].icon = new OpenLayers.Icon(icons.url + icons[i].url,icons.iconSize,offset);
                        icons[i].lonLat = new OpenLayers.LonLat(icons[i].coordinates[0],icons[i].coordinates[1]);
                        icons[i].lonLat.transform(pWGS,pMP);
                        icons[i].marker = new OpenLayers.Marker(icons[i].lonLat,icons[i].icon);
                        icons[i].popup = new OpenLayers.Popup.FramedCloud("Info",
                                            icons[i].lonLat,
                                                          null,
                                                icons[i].popupInfo,
                                            icons[i].icon,
                                                true,
                                            null
                                            );
                        icons[i].onClick = function(e) { map.addPopup(this.popup); this.popup.show(); }
                        icons[i].marker.events.register("mousedown", icons[i], function(e) { this.onClick(e)} )
                        layerMarkers.addMarker(icons[i].marker);
                    }
                }
                map.addLayer(layerMarkers);

                //zoom and center to the track layouts
                map.zoomToExtent(feature.geometry.getBounds());

            }
        </script>

        </head>
        <!-- body.onload is called once the page is loaded (call the 'init' function) -->
        <body onload="init();">
            <!-- define a DIV into which the map will appear. Make it take up the whole window -->
            <div style="width:100%; height:100%" id="map"></div>
        </body>
        </html>
        '''
        file = fileUtils(self.htmlfile,content)
        file.run()
        logging.debug("<<")

    def createErrorHtml(self,errMsg=None):
        logging.debug(">>")
        errMsg = errMsg or ''       # convert None to empty string
        content = '''
        <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
        <html xmlns="http://www.w3.org/1999/xhtml"  xmlns:v="urn:schemas-microsoft-com:vml">
            <body>
            No GPX Data (''' + str(errMsg) + ''')
            </body>
        </html>
        '''
        file = fileUtils(self.htmlfile,content)
        file.run()
        logging.debug("<<")




# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net
# vud1@grupoikusnet.com
# Jakinbidea & Grupo Ikusnet Developer

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import logging
import sys, traceback, commands
import datetime
try:
    from sqlite3 import dbapi2 as sqlite
except ImportError:
    logging.error('Not able to find sqlite2 module (new in python 2.5)')
    from pysqlite2 import dbapi2 as sqlite
    logging.info('Using pysqlite2 module to access DB. Think about upgrading to python 2.5!')
    
class Sql:
    def __init__(self,host=None, ddbb = None, user = None, password = None, configuration = None):
        self.db = None
        if ddbb == 'memory':
            self.ddbb = ':memory:'
            self.url = 'sqlite://'
        else:
            confdir = configuration.confdir
            self.ddbb = "%s/pytrainer.ddbb" %confdir
            self.url = "sqlite:///" + self.ddbb
        
    def get_connection_url(self):
        return self.url
    
    def connect(self):
        #si devolvemos 1 ha ido todo con exito
        self.db = sqlite.connect(self.ddbb)
        return (True, "OK")
        #probamos si estan las tablas creadas, y sino.. las creamos
        '''try: 
            self.select("records","id_record","1=1 limit 0,1")
        except:
            self.createTables()
        return 1'''

    def disconnect(self):
        self.db.close()
    
    def createDDBB(self):
        pass
        
    def getTableList(self):
        tmpList = self.select("sqlite_master","name", "type IN ('table','view') AND name NOT LIKE 'sqlite_%' ORDER BY name")
        # The instruction above returns a list of tuples, going for a simple list
        newList = []
        for entry in tmpList:
            newList.append(entry[0])
        return newList

    def createTableDefault(self,tableName,columns):
        '''22.11.2009 - dgranda
        Creates a new table in database given name and column name and data types. New in version 1.7.0
        args:
            tableName - string with name of the table
            columns - dictionary containing column names and data types coming from definition
        returns: none'''
        logging.debug('>>')
        logging.info('Creating '+str(tableName)+' table with default values')
        logging.debug('Columns definition: '+str(columns))
        cur = self.db.cursor()
        sql = 'CREATE TABLE %s (' %(tableName)
        for entry in columns:
            sql += '%s %s,' %(entry,columns[entry])
        # Removing trailing comma
        sql = sql.rstrip(',')
        sql = sql+");"
        logging.debug('SQL sentence: '+str(sql))
        cur.execute(sql)
        logging.debug('<<')
              
    def insert(self,table, cells, values):
        logging.debug('>>')
        cur = self.db.cursor()  
        val = values
        count = 0
        string = ""
        for i in val:
            if count>0:
                string+=","
            string+= self._to_sql_value(i)
            count = count+1
        sql = "insert into %s (%s) values (%s)"  %(table,cells,string)
        logging.debug('SQL sentence: '+str(sql))
        cur.execute(sql)
        self.db.commit()
        logging.debug('<<')
        
    def _to_sql_value(self, value):
        logging.debug('>>')
        logging.debug('Value: %s | type: %s ' %(value,type(value)))
        if value == None:
            return "null"
        elif type(value) in [str, unicode]:
            return "'%s'" % value
        elif type(value) == datetime.datetime:
            return value.strftime("'%Y-%m-%d %H:%M:%S%z'")
        elif type(value) == datetime.date:
            return value.strftime("'%Y-%m-%d'")
        else:
            return str(value)
        logging.debug('<<')

    def freeExec(self,sql):
        cur = self.db.cursor()
        cur.execute(sql)
        retorno = []
        for row in cur:
            retorno.append(row)
        self.db.commit()
        return retorno

    def delete(self,table,condition):
        cur = self.db.cursor()  
        sql = "delete from %s where %s"  %(table,condition)
        cur.execute(sql)
        self.db.commit()

    def update(self,table,cells,values, condition):
        cur = self.db.cursor()  
        cells = cells.split(",")
        count = 0
        string = ""
        for val in values:
            if count>0:
                string+=","
            string += """%s=%s """ %(cells[count], self._to_sql_value(values[count]))
            count = count+1

        string +=" where %s" %condition
        sql = "update %s set %s" %(table,string)
        cur.execute(sql)
        self.db.commit()

    def select(self,table,cells,condition, mod=None):
        cur = self.db.cursor()
        sql = "select %s from %s" %(cells,table)
        if condition is not None:
            sql = "%s where %s" % (sql, condition)
        if mod is not None:
            sql = "%s %s" % (sql, mod)
        logging.debug('SQL sentence: %s' % sql)
        cur.execute(sql)
        retorno = []
        for row in cur:
            retorno.append(row)
        return retorno

    def retrieveTableInfo(self,tableName):
        cur = self.db.cursor()
        sql = "PRAGMA table_info(%s);" %tableName
        cur.execute(sql)
        tableInfo = []
        for row in cur:
            tableInfo.append(row)
        return tableInfo

    def addColumn(self,tableName,columnName,dataType):
        sql = "alter table %s add %s %s" %(tableName,columnName,dataType)
        logging.debug("Trying SQL: %s" % sql)
        try:
            self.freeExec(sql)
        except:
            logging.error('Not able to add/change column '+columnName+' to table '+tableName)
            traceback.print_exc()


    def createDatabaseBackup(self):
        logging.info("Creating compressed copy of current DB")
        logging.debug('Database path: '+str(self.ddbb))
        result = commands.getstatusoutput('gzip -c '+self.ddbb+' > '+self.ddbb+'_`date +%Y%m%d_%H%M`.gz')
        if result[0] != 0:
            raise Exception, "Copying current database does not work, error #"+str(result[0])
        logging.info('Database backup successfully created')



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net
# Modified by dgranda

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import logging
import os, sys, commands
import StringIO
from lxml import etree

class gant():
	def __init__(self, parent = None, data_path = None):
		self.parent = parent
		self.pytrainer_main = parent.parent
		self.tmpdir = self.pytrainer_main.profile.tmpdir
		self.main_data_path = data_path
		self.data_path = os.path.dirname(__file__)

	def getName(self):
		return _("Gant")

	def getVersion(self):
		outstatus = commands.getstatusoutput('which gant')
		if outstatus[0] == 0: #Found gant in path 
			path = outstatus[1]
			return path
		else:
			return None
		return None

	def getSourceLocation(self):
		return "http://cgit.get-open.com/cgit.cgi/gant/"

	def deviceExists(self):
		return True

	def isPresent(self):
		if self.getVersion():
			return True
		else:
			return False



#!/usr/bin/env python
import os #, stat, sys
import logging
import gtk
from lxml import etree
from pytrainer.lib.srtmlayer import SrtmLayer

class fixelevation:
    _data = None
    _srtm = SrtmLayer()

    def __init__(self, parent = None, pytrainer_main = None, conf_dir = None, options = None):
        self.parent = parent
        self.pytrainer_main = pytrainer_main
        self.options = options
        self.conf_dir = conf_dir

    def run(self, aid, activity=None):  #TODO Convert to use activity...
        #print activity
        logging.debug(">>")
        gpx_file = "%s/gpx/%s.gpx" % (self.conf_dir, aid)
        ele_fixed = True
        
        if os.path.isfile(gpx_file):
            # Backup original raw data as *.orig.gpx
            #orig_file = open(gpx_file, 'r')
            #orig_data = orig_file.read()
            #orig_file.close()
            #backup_file = open("%s/gpx/%s.orig.gpx" % (self.conf_dir, id), 'w')
            #backup_file.write(orig_data)
            #backup_file.close()
            #GPX file is ok and found, so open it
            logging.debug("ELE GPX file: %s found, size: %d" % (gpx_file, os.path.getsize(gpx_file)))
            
            """
            Parse GPX file to ElementTree instance.
            """
            self._data = etree.parse(gpx_file)
            self._xmlns = self._data.getroot().nsmap[None]
            nsmap = self._data.getroot().nsmap
            pyt_ns = "http://sourceforge.net.project/pytrainer/GPX/0/1"
            PYTRAINER = "{%s}" % pyt_ns
            self._trkpt_path = '{%s}trk/{%s}trkseg/{%s}trkpt' % (self._xmlns, self._xmlns, self._xmlns)

            def addExt(trackpoint, ele_new):
                #Add new elevation to extension tag
                '''
                <extensions>
                    <pytrainer:ele method="srtm_bilinear">31.1</pytrainer:ele>
                </extensions>
                '''
                ext = etree.Element("extensions")
                py_ele = etree.SubElement(ext, PYTRAINER + "ele", method="srtm_bilinear")
                py_ele.text = str(ele_new)
                trackpoint.append(ext)

            """
            Replace elevation from GPX by data from SRTM.
            TODO (Arnd) make a function within class fixelevation out of this for better reuse
            """
            trackpoints = self._data.findall(self._trkpt_path)
            for trkpt in trackpoints:
                lat = float(trkpt.attrib['lat'])
                lon = float(trkpt.attrib['lon'])
            
                ele = trkpt.find('{%s}ele' % self._xmlns)
                ele_new = self._srtm.get_elevation(lat, lon)

                if not ele_new:
                    ele_fixed = False
                    break 

                addExt(trkpt, ele_new)
                
            if not ele_fixed:
                # Try Google maps elevation API
                import cjson, urllib2, math
                steps = len(trackpoints) / 300

                path = ''
                lat_prev, long_prev = 0, 0
                t = 0
                for t in xrange(0,len(trackpoints),steps):
                    lat = float(trackpoints[t].attrib['lat'])
                    lon = float(trackpoints[t].attrib['lon'])
                    encoded_lat, lat_prev = encode_coord(lat, lat_prev)
                    encoded_long, long_prev = encode_coord(lon, long_prev)
                    path += encoded_lat + encoded_long
                    t += 1
                url = "http://maps.googleapis.com/maps/api/elevation/json?sensor=true&samples=%d&path=enc:" % int((len(trackpoints) / steps))
                url += path
 
                try:
                    google_ele = cjson.decode(urllib2.urlopen(url).read())
                    if google_ele['status'] == "OK":
                        t_idx = 0
                        ele_points = len(google_ele['results'])
                        for ele_new in xrange(0,ele_points):
                            addExt(trackpoints[t_idx], google_ele['results'][ele_new]['elevation'])
                            for intermediate in xrange(ele_new+1, ele_new+steps):
                                if intermediate<len(trackpoints):
                                    if ele_new==ele_points-1:
                                        calculated = google_ele['results'][ele_new]['elevation']
                                    else:
                                        ele1 = google_ele['results'][ele_new]['elevation']
                                        ele2 = google_ele['results'][ele_new+1]['elevation']
                                        calculated = (ele1 * (intermediate-ele_new)  + ele2 * (steps - (intermediate-ele_new))) / steps
                                    t_idx += 1
                                    addExt(trackpoints[t_idx], calculated)
                            t_idx += 1
                        ele_fixed = True
                except urllib2.HTTPError:
                    pass

            if ele_fixed:
                # Write out to original *.gpx.                         
                self._data.write( gpx_file, 
                                encoding=self._data.docinfo.encoding, 
                                xml_declaration=True, 
                                pretty_print=False)
                res_msg = "Elevation has been fixed."
                #TODO Expire activity out of pool - so get updated info
                self.pytrainer_main.activitypool.remove_activity(aid)
            else:
                res_msg = "Elevation could not be fixed!"

            #Show the user the result
            md = gtk.MessageDialog(self.pytrainer_main.windowmain.window1, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, res_msg)
            md.set_title(_("Elevation Correction Complete"))
            md.set_modal(False)
            md.run()
            md.destroy()
            #TODO reload gpx data in main window 
            
        else:
            logging.error("ELE GPX file: %s NOT found!!!" % (gpx_file))
        logging.debug("<<")
        
def encode_coord(x, prev):
    val = int(x * 1e5)
    return encode_signed(val - prev), val

def encode_signed(n):
    tmp = n << 1
    if n < 0:
        tmp = ~tmp
    return encode_unsigned(tmp)

def encode_unsigned(n):
    b = []
    while n >= 32:
        b.append(n & 31)
        n = n >> 5
    b = [(c | 0x20) for c in b]
    b.append(n)
    b = [(i + 63) for i in b]
    return ''.join([chr(i) for i in b])




# -*- coding: iso-8859-1 -*-

#Copyright (C) Nathan Jones ncjones@users.sourceforge.net
#modified by dgranda on behalf of Debian bug #587997 (trac ticket #120)

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import os
import sys
    
def get_platform():
    if os.name == "posix":
        return _Linux() #although not true, not changing original return name to avoid side effects
    elif os.name == "nt":
        return _Windows()
    else:
        print "Unsupported os.name: %s." % os.name
        sys.exit(1)
        
class _Platform(object):
    
    def get_default_conf_dir(self):
        """Get the path to the default configuration directory for the platform."""
        return self._home_dir + "/" + self._conf_dir_name

    def get_default_data_path(self):
        """Get the path to the default data directory for the platform."""
        return self._data_path

class _Linux(_Platform):
    
    def __init__(self):
        self._home_dir = os.environ['HOME']
        self._conf_dir_name = ".pytrainer"
        self._data_path = "/usr/share/pytrainer/"

class _Windows(_Platform):
    
    def __init__(self):
        self._home_dir = os.environ['USERPROFILE']
        self._conf_dir_name = "pytrainer"
        self._data_path = os.getcwd() + os.sep



# -*- coding: iso-8859-1 -*-

#Copyright (C) Nathan Jones ncjones@users.sourceforge.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

class Color(object):
    
    """A color represented as a 24-bit RGB value."""
    
    def __init__(self, rgb_val=0):
        rgb_val_int = int(rgb_val)
        if rgb_val_int < 0:
            raise ValueError("RGB value must not be negative.")
        if rgb_val_int > 0xffffff:
            raise ValueError("RGB value must not be greater than 0xffffff.")
        self._rgb_val = rgb_val_int
        
    def _get_rgb_val(self):
        return self._rgb_val
    
    rgb_val = property(_get_rgb_val)
        
    def _get_rgba_val(self):
        return self._rgb_val << 8
    
    rgba_val = property(_get_rgba_val)
    
    def to_hex_string(self):
        return "{0:06x}".format(self._rgb_val)
    
def color_from_hex_string(hex_string):
    return Color(int(hex_string, 16))



# -*- coding: iso-8859-1 -*-

#Copyright (C) Nathan Jones ncjones@users.sourceforge.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import unittest
from mock import Mock
from pytrainer.upgrade.data import InstalledData, DataState, DataInitializationException
import pytrainer.upgrade.context

class InstalledDataTest(unittest.TestCase):
    
    def setUp(self):
        self._mock_migratable_db = Mock()
        self._mock_ddbb = Mock()
        self._mock_version_provider = Mock()
        self._mock_upgrade_context = Mock()
        self._installed_data = InstalledData(self._mock_migratable_db, self._mock_ddbb, self._mock_version_provider, self._mock_upgrade_context)
        
    def test_get_version_should_return_migrate_version_when_available(self):
        self._mock_migratable_db.get_version.return_value = 1
        self.assertEquals(1, self._installed_data.get_version())
        
    def test_get_version_should_return_legacy_version_when_available(self):
        self._mock_migratable_db.is_versioned.return_value = False
        self._mock_version_provider.get_legacy_version.return_value = 2
        self.assertEquals(2, self._installed_data.get_version())
        
    def test_get_version_should_return_none_when_no_existing_version(self):
        self._mock_migratable_db.is_versioned.return_value = False
        self._mock_version_provider.get_legacy_version.return_value = None
        self.assertEquals(None, self._installed_data.get_version())
        
    def test_get_state_should_return_current_when_data_version_equals_repository_version(self):
        self._mock_migratable_db.get_version.return_value = 1
        self._mock_migratable_db.get_upgrade_version.return_value = 1
        self.assertEquals(DataState.CURRENT, self._installed_data.get_state())

    def test_get_state_should_return_fresh_when_data_version_unavailable(self):
        self._mock_migratable_db.is_versioned.return_value = False
        self._mock_version_provider.get_legacy_version.return_value = None
        self.assertEquals(DataState.FRESH, self._installed_data.get_state())

    def test_get_state_should_return_stale_when_data_version_less_than_repository_version(self):
        self._mock_migratable_db.get_version.return_value = 1
        self._mock_migratable_db.get_upgrade_version.return_value = 2
        self.assertEquals(DataState.STALE, self._installed_data.get_state())

    def test_get_state_should_return_legacy_when_data_version_is_legacy(self):
        self._mock_migratable_db.is_versioned.return_value = False
        self._mock_version_provider.get_legacy_version.return_value = 1
        self.assertEquals(DataState.LEGACY, self._installed_data.get_state())

    def test_get_state_should_raise_error_when_version_too_large(self):
        self._mock_migratable_db.is_versioned.return_value = True
        self._mock_migratable_db.get_version.return_value = 2
        self._mock_migratable_db.get_upgrade_version.return_value = 1
        try:
            self._installed_data.get_state()
        except DataInitializationException:
            pass
        else:
            self.fail()
            
    def test_get_available_version_returns_migratable_db_upgrade_version(self):
        self._mock_migratable_db.get_upgrade_version.return_value = 1
        self.assertEquals(1, self._installed_data.get_available_version())
        
    def test_is_versioned_returns_migratable_db_is_versioned(self):
        self._mock_migratable_db.is_versioned.return_value = True
        self.assertEquals(True, self._installed_data.is_versioned())
        
    def test_initialize_version_versions_migratable_db(self):
        self._installed_data.initialize_version(1)
        self._mock_migratable_db.version.assert_called_with(1)
        
    def test_initialize_creates_db_tables(self):
        self._installed_data.initialize()
        self.assertTrue(self._mock_ddbb.create_tables.called)
        
    def test_upgrade_upgrades_migratable_db(self):
        self._installed_data.upgrade()
        self.assertTrue(self._mock_migratable_db.upgrade.called)
        
    def test_upgrade_initializes_global_upgrade_context(self):
        self._installed_data.upgrade()
        self.assertEqual(self._mock_upgrade_context, pytrainer.upgrade.context.UPGRADE_CONTEXT)
        
    def test_update_to_current_delegates_to_data_state(self):
        mock_state = Mock()
        self._installed_data.get_state = Mock(return_value=mock_state)
        self._installed_data.update_to_current()
        self.assertTrue(mock_state.update_to_current.called)



# SimpleGladeApp.py
# Module that provides an object oriented abstraction to pygtk and libglade.
# Copyright (C) 2004 Sandino Flores Moreno

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

import os
import sys
import gtk
from pytrainer.environment import Environment

class SimpleBuilderApp(dict):
    def __init__(self, ui_filename):
        self._builder = gtk.Builder()
        env = Environment()
        file_path = os.path.join(env.glade_dir, ui_filename)
        self._builder.add_from_file(file_path)
        self._builder.connect_signals(self)
        self.new()

    def signal_autoconnect(self):
        signals = {}
        for attr_name in dir(self):
            attr = getattr(self, attr_name)
            if callable(attr):
                signals[attr_name] = attr
        self._builder.connect_signals(signals)

    def __getattr__(self, data_name):
        if data_name in self:
            data = self[data_name]
            return data
        else:
            widget = self._builder.get_object(data_name)
            if widget != None:
                self[data_name] = widget
                return widget
            else:
                raise AttributeError, data_name

    def __setattr__(self, name, value):
        self[name] = value

    def new(self):
        pass

    def on_keyboard_interrupt(self):
        pass

    def gtk_widget_show(self, widget, *args):
        widget.show()

    def gtk_widget_hide(self, widget, *args):
        widget.hide()

    def gtk_widget_grab_focus(self, widget, *args):
        widget.grab_focus()

    def gtk_widget_destroy(self, widget, *args):
        widget.destroy()

    def gtk_window_activate_default(self, widget, *args):
        widget.activate_default()

    def gtk_true(self, *args):
        return gtk.TRUE

    def gtk_false(self, *args):
        return gtk.FALSE

    def gtk_main_quit(self, *args):
        gtk.main_quit()

    def main(self):
        gtk.main()

    def quit(self, widget=None):
        gtk.main_quit()

    def run(self):
        try:
            self.main()
        except KeyboardInterrupt:
            self.on_keyboard_interrupt()

    def create_treeview(self,treeview,column_names):
        i=0
        for column_index, column_name in enumerate(column_names):
            column = gtk.TreeViewColumn(column_name, gtk.CellRendererText(), text=column_index)
            column.set_resizable(True)
            if i==0:
                column.set_visible(False)
            column.set_sort_column_id(i)
            treeview.append_column(column)




# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net
#Modified by dgranda

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import os
import shutil
import logging
import traceback

from gui.windowrecord import WindowRecord
from gui.dialogselecttrack import DialogSelectTrack
from lib.xmlUtils import XMLParser
from lib.date import Date, time2second
from lib.gpx import Gpx
from pytrainer.core.equipment import EquipmentService
from pytrainer.core.sport import Sport

class Record:
	def __init__(self, sport_service, data_path = None, parent = None):
		logging.debug('>>')
		self._sport_service = sport_service
		self.parent = parent
		self.pytrainer_main = parent
		self._equipment_service = EquipmentService(self.pytrainer_main.ddbb)
		self.data_path = data_path
		logging.debug('setting date...')
		self.date = Date()
		logging.debug('<<')

	def newRecord(self, date, title=None, distance=None, time=None, upositive=None, unegative=None, bpm=None, calories=None, comment=None):
		logging.debug('>>')
		sports = self._sport_service.get_all_sports()
		self.recordwindow = WindowRecord(self._equipment_service, self.data_path, sports, self, self.format_date(date), title, distance, time, upositive, unegative, bpm, calories, comment)
		self.recordwindow.run()
		logging.debug('<<')

	def newMultiRecord(self, activities):
		logging.debug('>>')
		sports = self._sport_service.get_all_sports()
		self.recordwindow = WindowRecord(self._equipment_service, self.data_path, sports, parent=self, windowTitle=_("Modify details before importing"))
		self.recordwindow.populateMultiWindow(activities)
		self.recordwindow.run()
		return self.recordwindow.getActivityData()
		logging.debug('<<')

	def editRecord(self,id_record):
		logging.debug('>>')
		activity = self.pytrainer_main.activitypool.get_activity(id_record)
		record_equipment = self.get_record_equipment(id_record)
		sports = self._sport_service.get_all_sports()
		self.recordwindow = WindowRecord(self._equipment_service, self.data_path, sports, self, None, windowTitle=_("Edit Entry"), equipment=record_equipment)
		self.recordwindow.setValuesFromActivity(activity)
		logging.debug('launching window')
		self.recordwindow.run()
                self.pytrainer_main.refreshMainSportList()
		logging.debug('<<')

	def removeRecord(self,id_record):
		logging.debug('>>')
		record = self.pytrainer_main.ddbb.delete("records", "id_record=\"%s\"" %id_record)
		laps = self.pytrainer_main.ddbb.delete("laps", "record=\"%s\"" %id_record)
		logging.debug('removed record '+str(id_record)+' (and associated laps) from DB')
		gpxfile = self.pytrainer_main.profile.gpxdir+"/%d.gpx"%int(id_record)
		if os.path.isfile(gpxfile):
			os.remove(gpxfile)
			logging.debug('removed gpxfile '+gpxfile)
		logging.debug('<<')

	def pace_to_float(self, value):
		'''Take a mm:ss or mm.ss and return float'''
		try:
			value = float(value)
		except:
			if ":" in value: # 'mm:ss' found
				mins, sec = value.split(":")
				value = float(mins + "." + "%02d" %round(int(sec)*5/3))
			elif "," in value:
				value = float(value.replace(',','.'))
			else:
				logging.error("Wrong value provided: %s" %value)
				value = None
		return value

	def pace_from_float(self, value, fromDB=False):
		'''Helper to generate mm:ss from float representation mm.ss (or mm,ss?)'''
		#Check that value supplied is a float
		try:
			_value = "%0.2f" % float(value)
		except ValueError:
			_value = str(value)
		if fromDB:  # paces in DB are stored in mixed format -> 4:30 as 4.3 (NOT as 4.5 aka 'decimal')
			pace = _value
		else:
			mins, sec_dec = _value.split(".")
			pace = mins + ":" + "%02d" %round(int(sec_dec)*3/5)
		return pace

	def _formatRecordNew (self, list_options):
		"""20.07.2008 - dgranda
		New records handle date_time_utc field which is transparent when updating, so logic method has been splitted
		args: list with keys and values without valid format
		returns: keys and values matching DB schema"""
		logging.debug('>>')
		time = time2second(list_options["rcd_time"])
		average = self.parseFloatRecord(list_options["rcd_average"])
		keys= "date,sport,distance,time,beats,comments,average,calories,title,upositive,unegative,maxspeed,maxpace,pace,maxbeats,date_time_utc,date_time_local, duration"
		if (list_options["rcd_beats"] == ""):
			list_options["rcd_beats"] = 0

		#retrieving sport id (adding sport if it doesn't exist yet)
		sport_id = self.getSportId(list_options["rcd_sport"],add=True)

		values= (
			list_options["rcd_date"],
			sport_id,
			self.parseFloatRecord(list_options["rcd_distance"]),
			time,
			self.parseFloatRecord(list_options["rcd_beats"]),
			list_options["rcd_comments"],
			average,
			self.parseFloatRecord(list_options["rcd_calories"]),
			list_options["rcd_title"],
			self.parseFloatRecord(list_options["rcd_upositive"]),
			self.parseFloatRecord(list_options["rcd_unegative"]),
			self.parseFloatRecord(list_options["rcd_maxvel"]),
			self.pace_to_float(list_options["rcd_maxpace"]),
			self.pace_to_float(list_options["rcd_pace"]),
			self.parseFloatRecord(list_options["rcd_maxbeats"]),
			list_options["date_time_utc"],
			list_options["date_time_local"],
			time,
			)
		logging.debug('<<')
		return keys,values

	def insertRecord(self, list_options, laps=None, equipment=None):
		logging.debug('>>')
		#Create entry for activity in records table
		if list_options is None:
			logging.info('No data provided, abort adding entry')
			return None
		logging.debug('list_options: '+str(list_options))
		cells,values = self._formatRecordNew(list_options)
		self.pytrainer_main.ddbb.insert("records",cells,values)
		logging.debug('DB updated: '+str(cells)+' | '+str(values))
		id_record = self.pytrainer_main.ddbb.lastRecord("records")
		#Create entry(s) for activity in laps table
		if laps is not None:
			for lap in laps:
				lap['record'] = id_record #Add reference to entry in record table
				lap_keys = ", ".join(map(str, lap.keys()))
				lap_values = lap.values()
				self.insertLaps(lap_keys,lap.values())
		if equipment is not None:
			for equipment_id in equipment:
				self._insert_record_equipment(id_record, equipment_id)
		gpxOrig = list_options["rcd_gpxfile"]
		if os.path.isfile(gpxOrig):
			gpxDest = self.pytrainer_main.profile.gpxdir
			gpxNew = gpxDest+"/%d.gpx"%id_record
			#Leave original file in place...
			#shutil.move(gpxOrig, gpxNew)
			#logging.debug('Moving '+gpxOrig+' to '+gpxNew)
			shutil.copy(gpxOrig, gpxNew)
			logging.debug('Copying '+gpxOrig+' to '+gpxNew)
		#self.parent.refreshListRecords()
		logging.debug('<<')
		return self.pytrainer_main.ddbb.lastRecord("records")

	def insertNewRecord(self, gpxOrig, entry): #TODO consolidate with insertRecord
		"""29.03.2008 - dgranda
		Moves GPX file to store destination and updates database
		args: path to source GPX file"""
		logging.debug('--')
		(list_options, gpx_laps) = self.summaryFromGPX(gpxOrig, entry)
		if list_options is None:
			return None
		return self.insertRecord(list_options, laps=gpx_laps)

	def lapsFromGPX(self, gpx):
		logging.debug('>>')
		laps = []
		gpxLaps = gpx.getLaps()
		for lap in gpxLaps:
			lap_number = gpxLaps.index(lap)
			tmp_lap = {}
			tmp_lap['record'] = ""
			tmp_lap['lap_number'] = lap_number
			tmp_lap['elapsed_time'] = lap[0]
			tmp_lap['distance'] = lap[4]
			tmp_lap['start_lat'] = lap[5]
			tmp_lap['start_lon'] = lap[6]
			tmp_lap['end_lat'] = lap[1]
			tmp_lap['end_lon'] = lap[2]
			tmp_lap['calories'] = lap[3]
			tmp_lap['intensity'] = lap[7]
			tmp_lap['avg_hr'] = lap[8]
			tmp_lap['max_hr'] = lap[9]
			tmp_lap['max_speed'] = lap[10]
			tmp_lap['laptrigger'] = lap[11]
			tmp_lap['comments'] = ""
			laps.append(tmp_lap)
		logging.debug('<<')
		return laps

	def hrFromLaps(self, laps):
		logging.debug('>>')
		lap_avg_hr = 0 
		lap_max_hr = 0
		total_duration = 0
		ponderate_hr = 0;
		for lap in laps:
			if (lap['max_hr'] is None):
				lap['max_hr'] = 0
			if (lap['avg_hr'] is None):
				lap['avg_hr'] = 0
			if int(lap['max_hr']) > lap_max_hr:
				lap_max_hr = int(lap['max_hr'])
			total_duration = total_duration + float(lap['elapsed_time'])
			ponderate_hr = ponderate_hr + float(lap['elapsed_time'])*int(lap['avg_hr'])
			logging.debug("Lap number: %s | Duration: %s | Average hr: %s | Maximum hr: %s" % (lap['lap_number'], lap['elapsed_time'], lap['avg_hr'], lap['max_hr']))
		lap_avg_hr = int(round(ponderate_hr/total_duration)) # ceil?, floor?, round?
		logging.debug('<<')
		return lap_avg_hr, lap_max_hr

	def summaryFromGPX(self, gpxOrig, entry):
		"""29.03.2008 - dgranda
		Retrieves info which will be stored in DB from GPX file
		args: path to source GPX file
		returns: list with fields and values, list of laps
		"""
		logging.debug('>>')
		gpx = Gpx(self.data_path,gpxOrig)
		distance, time, maxspeed, maxheartrate = gpx.getMaxValues()
		#if time == 0: #invalid record
		#	print "Invalid record"
		#	return (None, None)
		upositive,unegative = gpx.getUnevenness()
		if time > 0:
			speed = distance*3600/time
			time_hhmmss = [time//3600,(time/60)%60,time%60]
		else:
			speed = 0
			time_hhmmss = [0,0,0]
		summaryRecord = {}
		summaryRecord['rcd_gpxfile'] = gpxOrig
		summaryRecord['rcd_sport'] = entry[0]
		summaryRecord['rcd_date'] = gpx.getDate()
		summaryRecord['rcd_calories'] = gpx.getCalories()
		summaryRecord['rcd_comments'] = ''
		summaryRecord['rcd_title'] = ''
		summaryRecord['rcd_time'] = time_hhmmss #ToDo: makes no sense to work with arrays
		summaryRecord['rcd_distance'] = "%0.2f" %distance
		if speed == 0:
			summaryRecord['rcd_pace'] = "0"
		else:
			summaryRecord['rcd_pace'] = "%d.%02d" %((3600/speed)/60,(3600/speed)%60)
		if maxspeed == 0:
			summaryRecord['rcd_maxpace'] = "0"
		else:
			summaryRecord['rcd_maxpace'] = "%d.%02d" %((3600/maxspeed)/60,(3600/maxspeed)%60)
		summaryRecord['rcd_average'] = speed
		summaryRecord['rcd_maxvel'] = maxspeed
		summaryRecord['rcd_beats'] = gpx.getHeartRateAverage()
		summaryRecord['rcd_maxbeats'] = maxheartrate
		summaryRecord['rcd_upositive'] = upositive
		summaryRecord['rcd_unegative'] = unegative
		if entry[1]=="": # coming from new track dialog (file opening)												#TODO This if-else needs checking
			summaryRecord['date_time_utc'], summaryRecord['date_time_local'] = gpx.getStartTimeFromGPX(gpxOrig)		#
		else: # coming from GPS device																				#
			summaryRecord['date_time_utc'] = entry[1]																#
			summaryRecord['date_time_local'] = entry[1]																#
			print "#TODO fix record summaryRecord local and utc time..."											#
		logging.debug('summary: '+str(summaryRecord))
		laps = self.lapsFromGPX(gpx)
		# Heartrate data can't be retrieved if no trackpoints present, calculating from lap info
		lap_avg_hr, lap_max_hr = self.hrFromLaps(laps)
		logging.debug("HR data from laps. Average: %s | Maximum hr: %s" % (lap_avg_hr, lap_max_hr))
		if int(summaryRecord['rcd_beats']) > 0:
			logging.debug("Average heartbeat - Summary: %s | Laps: %s" % (summaryRecord['rcd_beats'], lap_avg_hr))
		else:
			logging.debug("No average heartbeat found, setting value (%s) from laps", lap_avg_hr)
			summaryRecord['rcd_beats'] = lap_avg_hr
		if int(summaryRecord['rcd_maxbeats']) > 0:
			logging.debug("Max heartbeat - Summary: %s | Laps: %s" % (summaryRecord['rcd_maxbeats'], lap_max_hr))
		else:
			logging.debug("No max heartbeat found, setting value (%s) from laps", lap_max_hr)
			summaryRecord['rcd_maxbeats'] = lap_max_hr
		logging.debug('<<')
		return summaryRecord, laps

	def updateRecord(self, list_options, id_record, equipment=None): # ToDo: update only fields that can change if GPX file is present
		logging.debug('>>')
		#Remove activity from pool so data is updated
		self.pytrainer_main.activitypool.remove_activity(id_record)
		gpxfile = self.pytrainer_main.profile.gpxdir+"/%d.gpx"%int(id_record)
		gpxOrig = list_options["rcd_gpxfile"]
		if os.path.isfile(gpxOrig):
			if gpxfile != gpxOrig:
				shutil.copy2(gpxOrig, gpxfile)
		else:
			if (list_options["rcd_gpxfile"]==""):
				logging.debug('Activity not based in GPX file') # ein?
		logging.debug('Updating bbdd')
		cells,values = self._formatRecordNew(list_options)
		self.pytrainer_main.ddbb.update("records",cells,values," id_record=%d" %int(id_record))
		if equipment is not None:
			self._update_record_equipment(id_record, equipment)
		self.pytrainer_main.refreshListView()
		logging.debug('<<')

	def parseFloatRecord(self,string):
		logging.debug('--')
		if string != "":
			try:
				return float(string.replace(",",","))
			except:
				return float(string)
		else:
			return 0

	def getrecordInfo(self,id_record):
		logging.debug('--')
		if id_record is None or id_record == "":
			return []
		return self.pytrainer_main.ddbb.select("records,sports",
					"sports.name,date,distance,time,beats,comments,average,calories,id_record,title,upositive,unegative,maxspeed,maxpace,pace,maxbeats,date_time_utc,date_time_local",
					"id_record=\"%s\" and records.sport=sports.id_sports" %id_record)

	def format_date(self, date):
		return date.strftime("%Y-%m-%d")

	def getrecordList(self,date, id_sport=None):
		logging.debug('--')
		if not id_sport:
			# outer join on sport id to workaround bug where sport reference is null on records from GPX import
			return self.pytrainer_main.ddbb.select("records left outer join sports on records.sport=sports.id_sports",
					"sports.name,date,distance,time,beats,comments,average,calories,id_record,maxspeed,maxbeats,date_time_utc,date_time_local,upositive,unegative",
					"date=\"%s\" " %self.format_date(date))
		else:
			return self.pytrainer_main.ddbb.select("records,sports",
					"sports.name,date,distance,time,beats,comments,average,calories,id_record,maxspeed,maxbeats,date_time_utc,date_time_local,upositive,unegative",
					"date=\"%s\" and sports.id_sports=\"%s\" and records.sport=sports.id_sports" %(self.format_date(date),id_sport))

	def getLaps(self, id_record):
		logging.debug('--')
		laps = self.pytrainer_main.ddbb.select("laps",
					"id_lap, record, elapsed_time, distance, start_lat, start_lon, end_lat, end_lon, calories, lap_number, intensity, max_speed, avg_hr, max_hr, laptrigger, comments",
					"record=\"%s\"" % id_record)
		if laps is None or laps == []:  #No laps stored - update DB
			logging.debug("No laps in DB for record %d" % id_record)
			#print ("No laps in DB for record %d" % id_record)
			gpx_dest = self.pytrainer_main.profile.gpxdir
			gpxfile = gpx_dest+"/%d.gpx"%id_record
			gpx = Gpx(self.data_path,gpxfile)
			laps = self.lapsFromGPX(gpx)
			if laps is not None:
				for lap in laps:
					lap['record'] = id_record #Add reference to entry in record table
					lap_keys = ", ".join(map(str, lap.keys()))
					lap_values = lap.values()
					self.insertLaps(lap_keys,lap.values())
			#Try to get lap info again #TODO? refactor
			laps = self.pytrainer_main.ddbb.select("laps",
					"id_lap, record, elapsed_time, distance, start_lat, start_lon, end_lat, end_lon, calories, lap_number, intensity, max_speed, avg_hr, max_hr, laptrigger, comments",
					"record=\"%s\"" % id_record)
		return laps

	def insertLaps(self, cells, values):
		logging.debug('--')
		logging.debug("Adding lap information: " + ", ".join(map(str, values)))
		self.pytrainer_main.ddbb.insert("laps",cells,values)
		
	def _insert_record_equipment(self, record_id, equipment_id):
		self.pytrainer_main.ddbb.insert("record_equipment", "record_id, equipment_id", [record_id, equipment_id])
		
	def _update_record_equipment(self, record_id, equipment_ids):
		self.pytrainer_main.ddbb.delete("record_equipment", "record_id={0}".format(record_id))
		for id in equipment_ids:
			self._insert_record_equipment(record_id, id)
			
	def get_record_equipment(self, record_id):
		record_equipment = []
		results = self.pytrainer_main.ddbb.select("record_equipment", "equipment_id", "record_id={0}".format(record_id))
		for row in results:
			id = row[0]
			equipment_item = self._equipment_service.get_equipment_item(id)
			record_equipment.append(equipment_item)
		return record_equipment

	def getrecordPeriod(self, date_range, sport=None):
		#TODO This is essentially the same as getrecordPeriodSport (except date ranges) - need to look at merging the two
		date_ini = self.format_date(date_range.start_date)
		date_end = self.format_date(date_range.end_date)
		tables = "records,sports"
		if not sport:
			condition = "date>=\"%s\" and date<=\"%s\" and records.sport=sports.id_sports" %(date_ini,date_end)
		else:
			condition = "date>=\"%s\" and date<=\"%s\" and records.sport=sports.id_sports and sports.id_sports=\"%s\"" %(date_ini,date_end, sport)

		return self.pytrainer_main.ddbb.select(tables,"date,distance,time,beats,comments,average,calories,maxspeed,maxbeats, sports.name,upositive,unegative", condition)

	def getrecordPeriodSport(self,date_ini, date_end,sport):
		if not sport:
			tables = "records"
			condition = "date>\"%s\" and date<\"%s\"" %(date_ini,date_end)
		else :
			tables = "records,sports"
			condition = "date>\"%s\" and date<\"%s\" and records.sport=sports.id_sports and sports.id_sports=\"%s\"" %(date_ini,date_end,sport)

		return self.pytrainer_main.ddbb.select(tables,
					"date,distance,time,beats,comments,average,calories,maxspeed,maxbeats,upositive,unegative",
					condition)
		
	def _get_sport(self, sport_name):
		return self._sport_service.get_sport_by_name(sport_name)

	def getSportMet(self,sport_name):
		"""Deprecated: use sport.met"""
		logging.debug('--')
		return self._get_sport(sport_name).met

	def getSportWeight(self,sport_name):
		"""Deprecated: use sport.weight"""
		logging.debug('--')
		return self._get_sport(sport_name).weight

	def getSportId(self, sport_name, add=None):
		"""Deprecated: use sport_service.get_sport_by_name()
		
		Get the id of a sport by name, optionally adding a new sport if
		none exists with the given name.
		arguments:
			sport_name: sport's name to get id for
			add: whether the sport should be added if not found
		returns: id for sport with given name or None"""
		if sport_name is None:
			return None
		sport = self._get_sport(sport_name)
		if sport is None:
			logging.debug("No sport with name: '%s'", str(sport_name))
			if add is not None:
				logging.debug("Adding sport '%s'", str(sport_name))
				new_sport = Sport()
				new_sport.name = unicode(sport_name)
				sport = self._sport_service.store_sport(new_sport)
		return None if sport is None else sport.id

	def getLastRecordDateString(self, sport_id = None):
		"""
		Retrieve date (string format) of last record stored in DB. It may select per sport
		"""
		logging.debug("--")
		#select date from records order by date desc limit 1;
		query_cond = None
		if sport_id is not None:
			query_cond = "sport = %s" % sport_id
		result = self.pytrainer_main.ddbb.select("records", "date", query_cond, "order by date desc limit 1")
        # It returns a list of tuples, so we take first element of list (tuple) and then select first element
		return result[0][0]

	def getAllrecord(self):
		"""
		Retrieve all record data (no lap nor equipment) stored in database. Initially intended for csv export
		arguments:
		returns: list of data sorted by date desc"""
		logging.debug('--')
		return self.pytrainer_main.ddbb.select("records,sports", "date_time_local,title,sports.name,distance,duration,average,maxspeed,pace,maxpace,beats,maxbeats,calories,upositive,unegative,comments",
            "sports.id_sports = records.sport","order by date_time_local asc")

	def getAllRecordList(self):
		logging.debug('--')
		return self.pytrainer_main.ddbb.select("records,sports",
			"date,distance,average,title,sports.name,id_record,time,beats,calories",
			"sports.id_sports = records.sport order by date desc")

	def getRecordListByCondition(self,condition):
		logging.debug('>>')
		if condition is None:
			return self.getAllRecordList()
		else:
			logging.debug("condition: %s" % condition)
			return self.pytrainer_main.ddbb.select("records,sports",
				"date,distance,average,title,sports.name,id_record,time,beats,calories",
				"sports.id_sports = records.sport and %s order by date desc" %condition)

	def getRecordDayList(self,date, id_sport=None):
		logging.debug('>>')
		logging.debug('Retrieving data for ' + str(date))
		# Why is looking for all days of the same month?
		if not id_sport:
			records = self.pytrainer_main.ddbb.select("records","date","date LIKE '" +str(date.year)+"-"+date.strftime("%m")+"-%'")
		else:
			records = self.pytrainer_main.ddbb.select("records","date","date LIKE \"%d-%0.2d-%%\" and sport=\"%s\"" %(date.year,date.month,id_sport))
		logging.debug('Found '+str(len(records))+' entries')
		day_list = []
		for i in records:
			record = str(i[0]).split("-")
			logging.debug('date:'+str(i[0]))
			day_list.append(record[2])
		logging.debug('<<')
		return day_list

	def actualize_fromgpx(self,gpxfile): #TODO remove? - should never have multiple tracks per GPX file
		logging.debug('>>')
		logging.debug('loading file: '+gpxfile)
		gpx = Gpx(self.data_path,gpxfile)
		tracks = gpx.getTrackRoutes()

		if len(tracks) == 1:
			logging.debug('Just 1 track')
			self._actualize_fromgpx(gpx)
		elif len(tracks) > 1:
			logging.debug('Found '+str(len(tracks))+' tracks')
			self._select_trkfromgpx(gpxfile,tracks)
		else:
			msg = _("pytrainer can't import data from your gpx file")
			from gui.warning import Warning
			warning = Warning(self.data_path)
			warning.set_text(msg)
			warning.run()
		logging.debug('<<')

	def _actualize_fromgpx(self, gpx):
		logging.debug('>>')
		distance, time, maxspeed, maxheartrate = gpx.getMaxValues()
		upositive,unegative = gpx.getUnevenness()
		heartrate = gpx.getHeartRateAverage()
		date = gpx.getDate()
		calories = gpx.getCalories()
		start_time = gpx.getStart_time()

		self.recordwindow.rcd_date.set_text(date)
		self.recordwindow.rcd_starttime.set_text(start_time)
		self.recordwindow.rcd_upositive.set_text(str(upositive))
		self.recordwindow.rcd_unegative.set_text(str(unegative))
		self.recordwindow.rcd_beats.set_text(str(heartrate))
		self.recordwindow.rcd_calories.set_text(str(calories))
		self.recordwindow.set_distance(distance)
		self.recordwindow.set_maxspeed(maxspeed)
		self.recordwindow.set_maxhr(maxheartrate)
		self.recordwindow.set_recordtime(time/60.0/60.0)
		self.recordwindow.on_calcavs_clicked(None)
		self.recordwindow.on_calccalories_clicked(None)
		self.recordwindow.rcd_maxpace.set_text("%d.%02d" %((3600/maxspeed)/60,(3600/maxspeed)%60))
		logging.debug('<<')

	def __actualize_fromgpx(self, gpxfile, name=None):
		logging.debug('>>')
		gpx = Gpx(self.data_path,gpxfile,name)
		self._actualize_fromgpx(gpx)
		logging.debug('<<')

	def _select_trkfromgpx(self,gpxfile,tracks):  #TODO remove? - should never have multiple tracks per GPX file
		logging.debug('>>')
		logging.debug('Track dialog '+ self.data_path +'|'+ gpxfile)
		selectrckdialog = DialogSelectTrack(self.data_path, tracks,self.__actualize_fromgpx, gpxfile)
		logging.debug('Launching window...')
		selectrckdialog.run()
		logging.debug('<<')

	def importFromGPX(self, gpxFile, sport):
		"""
		Add a record from a valid pytrainer type GPX file
		"""
		logging.debug('>>')
		entry_id = None
		if not os.path.isfile(gpxFile):
			logging.error("Invalid file: " +gpxFile)
		else:
			logging.info('Retrieving data from '+gpxFile)
			if not sport:
				sport = "import"
			entry = [sport,""]
			entry_id = self.insertNewRecord(gpxFile, entry)
			if entry_id is None:
				logging.error("Entry not created for file %s" % gpxFile)
			else:
				logging.info("Entry %d has been added" % entry_id)
		logging.debug('<<')
		return entry_id



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net
# Modified by dgranda

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import logging
import os
import traceback
from lxml import etree
from pytrainer.lib.date import getDateTime

from pytrainer.lib.xmlUtils import XMLParser

class garmintcxv2():
    def __init__(self, parent = None, data_path = None):
        if parent is not None:
            self.parent = parent
            self.pytrainer_main = parent.parent
            self.tmpdir = self.pytrainer_main.profile.tmpdir
        if data_path is not None:
            self.main_data_path = data_path
            self.data_path = os.path.dirname(__file__)
        self.xmldoc = None
        self.activitiesSummary = []
        self.activities = []

    def getXmldoc(self):
        ''' Function to return parsed xmlfile '''
        return self.xmldoc

    def getFileType(self):
        return _("Garmin training center database file version 2")

    def getActivitiesSummary(self):
        return self.activitiesSummary

    def getDetails(self, activity, startTime):
        logging.debug(">>")
        distance = 0
        duration = 0
        laps = activity.findall(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2}Lap")
        if laps:
            for lap in laps:
                lap_duration = float(lap.findtext(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2}TotalTimeSeconds"))
                lap_distance = float(lap.findtext(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2}DistanceMeters"))
                logging.debug("Lap distance (m): %f | duration (s): %f" % (lap_distance, lap_duration))
                distance += lap_distance
                duration += lap_duration
            hours = int(duration)//3600
            minutes = (int(duration)/60)%60
            seconds = int(duration)%60
            duration_hhmmss = "%02d:%02d:%02d" % (hours, minutes, seconds)       
            logging.debug("Activity distance (m): %f | duration (hh:mm:ss - s): %s - %f" % (distance, duration_hhmmss, duration))
        else:
            points = activity.findall(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2}Trackpoint")
            while True:
                lastPoint = points[-1]
                try:
                    distance = lastPoint.find(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2}DistanceMeters")
                    if distance is None:
                        points = points[:-1]
                        continue
                    time = lastPoint.find(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2}Time")
                    distance = distance.text
                    time = time.text
                    break
                except:
                    #Try again without the last point (i.e work from end until find time and distance)
                    points = points[:-1]
                    continue
            duration_hhmmss = getDateTime(time)[0]-startTime[0]
            logging.debug("Activity distance (m): %f | duration (hh:mm:ss): %s" % (distance, duration_hhmmss))
        logging.debug("<<")
        return float(distance), duration_hhmmss

    def validate(self, xmldoc, schema):
        logging.debug(">>")
        xmlschema_doc = etree.parse(self.main_data_path + schema)
        xmlschema = etree.XMLSchema(xmlschema_doc)
        logging.debug("<<")
        return xmlschema.validate(xmldoc)

    def buildActivitiesSummary(self):
        logging.debug(">>")
        self.activities = self.getActivities()
        for activity in self.activities:
            startTime = getDateTime(self.getStartTimeFromActivity(activity))
            inDatabase = self.inDatabase(startTime)
            sport = self.getSport(activity)
            distance, duration  = self.getDetails(activity, startTime)
            distance = distance / 1000.0
            self.activitiesSummary.append((self.activities.index(activity),
                                                                 inDatabase, 
                                                                 startTime[1].strftime("%Y-%m-%dT%H:%M:%S"), 
                                                                 "%0.2f" % distance , 
                                                                 str(duration), 
                                                                 sport,
                                                                 ))
        logging.debug("<<")

    def testFile(self, filename):
        '''Check if file is valid TCXv2 one and if yes, retrieve activities from it'''
        logging.debug('>>')
        logging.debug("Testing %s" %filename)
        result = False
        try:
            xmldoc = etree.parse(filename)
            valid_xml = self.validate(xmldoc, "schemas/GarminTrainingCenterDatabase_v2.xsd")
            if (valid_xml):
                logging.debug("Valid TCXv2 file (%s)" %filename)
                self.xmldoc = xmldoc
                self.buildActivitiesSummary()
                result = True
        except:
            logging.debug("Traceback: %s" % traceback.format_exc())
        logging.debug('<<')
        return result

    def getActivities(self):
        '''Function to return all activities in Garmin training center version 2 file'''
        logging.debug('>>')
        activities = self.xmldoc.findall(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2}Activity")
        logging.debug('<<')
        return activities

    def inDatabase(self, startTime):
        #comparing date and start time (sport may have been changed in DB after import)
        logging.debug('>>')
        result = False
        if startTime is not None:
            logging.info("Checking if activity from %s exists in db" % startTime[0]) # 2012-10-14 10:02:42+00:00
            time = startTime[0].strftime("%Y-%m-%dT%H:%M:%SZ")
            # No parent provided when unit testing (EAFP approach)
            try:
                if self.parent.parent.ddbb.select("records","*","date_time_utc=\"%s\"" % (time)):
                    result = True
            except AttributeError:
                logging.error("No parent attribute in current instance (testing?), skipping db check")
        else:
            logging.info("No start time provided, nothing to check")
        logging.debug('<<')
        return result

    def getSport(self, activity):
        try:
            sport = activity.get("Sport")
        except:
            sport = "import"
        return sport

    def getStartTimeFromActivity(self, activity):
        timeElement = activity.find(".//{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2}Id")
        if timeElement is None:
            return None
        else:
            return timeElement.text

    def getGPXFile(self, ID, file_id):
        """ Generate GPX file based on activity ID
            Returns (sport, GPX filename)
        """
        sport = None
        gpxFile = None
        activityID = int(ID)
        activitiesCount = len(self.activities)
        if activitiesCount > 0 and activityID < activitiesCount:
            gpxFile = "%s/garmin-tcxv2-%s-%d.gpx" % (self.tmpdir, file_id, activityID)
            activity = self.activities[int(activityID)]
            sport = self.getSport(activity)
            self.createGPXfile(gpxFile, activity)
        return sport, gpxFile  

    def createGPXfile(self, gpxfile, activity):
        """ Function to transform a Garmin Training Center v2 Track to a valid GPX+ file"""
        xslt_doc = etree.parse(self.data_path+"/translate_garmintcxv2.xsl")
        transform = etree.XSLT(xslt_doc)
        xml_doc = activity
        result_tree = transform(xml_doc)
        result_tree.write(gpxfile, xml_declaration=True, encoding='UTF-8')




from pytrainer.lib import gpx
from pytrainer.upgrade.context import UPGRADE_CONTEXT
from sqlalchemy.sql.expression import text
import logging
import os.path
import sqlalchemy

#	lap info added in version 1.9.0
def upgrade(migrate_engine=None):
    if migrate_engine is None:
        # sqlalchemy-migrate 0.5.4 does not provide migrate engine to upgrade scripts
        migrate_engine = sqlalchemy.create_engine(UPGRADE_CONTEXT.db_url)
    logging.info("Populating laps details columns")
    resultset = migrate_engine.execute(text("select distinct record from laps where intensity is null"))
    record_ids = []
    for row in resultset:
        record_ids.append(row["record"])
    resultset.close()
    for record_id in record_ids:
        gpx_file = UPGRADE_CONTEXT.conf_dir + "/gpx/{0}.gpx".format(record_id)
        if os.path.isfile(gpx_file):
            gpx_record = gpx.Gpx(filename=gpx_file)
            populate_laps_from_gpx(migrate_engine, record_id, gpx_record)

def populate_laps_from_gpx(migrate_engine, record_id, gpx_record):
        resultset = migrate_engine.execute(text("select id_lap from laps where record=:record_id" ), record_id=record_id)
        lap_ids = []
        for row in resultset:
            lap_ids.append(row["id_lap"])
        resultset.close()
        if(len(lap_ids) > 0):
            logging.info("Populating laps from GPX for record %s" , record_id)
        for lap_id, gpx_lap in zip(lap_ids, gpx_record.getLaps()):
            populate_lap_from_gpx(migrate_engine, lap_id, gpx_lap)
        
def populate_lap_from_gpx(migrate_engine, lap_id, gpx_lap):
    logging.info("Populating lap details from GPX for lap %s" , lap_id)
    migrate_engine.execute(text("""update laps set
                                    intensity=:intensity,
                                    avg_hr=:avg_heart_rate,
                                    max_hr=:max_heart_rate,
                                    max_speed=:max_speed,
                                    laptrigger=:lap_trigger 
                                    where id_lap=:lap_id"""), 
                                    intensity=gpx_lap[7],
                                    avg_heart_rate=gpx_lap[8],
                                    max_heart_rate=gpx_lap[9],
                                    max_speed=float(gpx_lap[10]), 
                                    lap_trigger=gpx_lap[11],
                                    lap_id=lap_id)
    
# work around a migrate bug
try:
    import migrate.versioning.exceptions as ex1
    import migrate.changeset.exceptions as ex2
    ex1.MigrateDeprecationWarning = ex2.MigrateDeprecationWarning
except:
    pass



# -*- coding: iso-8859-1 -*-

#Copyright (C) Nathan Jones ncjones@users.sourceforge.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import datetime
from subprocess import Popen, PIPE

class DateRange(object):
    
    """A date range consisting of a start date and an end date."""
    
    def __init__(self, start_date, end_date):
        if not isinstance(start_date, datetime.date):
            raise TypeError("Start date must be datetime.date, not {0}.".format(type(start_date).__name__))
        if not isinstance(end_date, datetime.date):
            raise TypeError("End date must be datetime.date, not {0}.".format(type(start_date).__name__))
        if cmp(start_date, end_date) > 0:
            raise ValueError("End date cannot be before start date.")
        self._start_date = start_date
        self._end_date = end_date
        
    @property
    def start_date(self):
        return self._start_date
    
    @property
    def end_date(self):
        return self._end_date
    
    def __str__(self):
        fmt = "%Y%m%d"
        return self._start_date.strftime(fmt) + "-" + self._end_date.strftime(fmt)

    @classmethod
    def for_week_containing(clazz, date):
        """Get the date range for the week containing the given date.
        
        The date range will start on the first day of the week and end on the
        last day of the week. The week start and end days are locale dependent.
        
        Args:
            date (datetime.date): a date within the week to get the date range for.
        Returns:
            (DateRange): the date range for a week.
        """
        day_of_week = (int(date.strftime("%w")) - first_day_of_week()) % 7
        date_start = date + datetime.timedelta(days = 0 - day_of_week)
        date_end = date + datetime.timedelta(days = 6 - day_of_week)
        return DateRange(date_start, date_end)
    
    @classmethod
    def for_month_containing(clazz, date):
        """Get the date range for the month containing the given date.
        
        The date range will start on the first day of the month and end on the
        last day of the month.
        
        Args:
            date (datetime.date): a date within the month to get the date range for.
        Returns:
            (DateRange): the date range for a month.
        """
        date_start = datetime.date(date.year, date.month, 1)
        next_month = date.month + 1
        next_month_year = date.year
        if (next_month == 13):
            next_month = 1
            next_month_year += 1
        date_end = datetime.date(next_month_year, next_month, 1) - datetime.timedelta(days=1)
        return DateRange(date_start, date_end)
    
    @classmethod
    def for_year_containing(clazz, date):
        """Get the date range for the year containing the given date.
        
        The date range will start on the first day of the year and end on the
        last day of the year.
        
        Args:
            date (datetime.date): a date within the year to get the date range for.
        Returns:
            (DateRange): the date range for a year.
        """
        date_start = datetime.date(date.year, 1, 1)
        date_end = datetime.date(date.year, 12, 31)
        return DateRange(date_start, date_end)
    
def first_day_of_week():
    """Determine the first day of the week for the system locale.
    
    If the first day of the week cannot be determined then Sunday is assumed.
    
    Returns (int): a day of the week; 0: Sunday, 1: Monday, 6: Saturday.
    """
    # Python's locale does not have a first day of week so make a system call
    # http://blogs.gnome.org/patrys/2008/09/29/how-to-determine-the-first-day-of-week/
    CMD=("locale", "first_weekday", "week-1stday")
    process = Popen(CMD, stdout=PIPE, stderr=PIPE)
    stdout, stderr = process.communicate()
    if  process.returncode != 0:
        return 0
    results = stdout.split("\n")
    if len(results) < 2:
        return 0
    day_delta = datetime.timedelta(days=int(results[0]) - 1)
    base_date = datetime.datetime.strptime(results[1], "%Y%m%d")
    first_day = base_date + day_delta
    return int(first_day.strftime("%w"))



#Copyright (C) Arto Jantunen <viiru@iki.fi>

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import unittest
from datetime import datetime
from mock import Mock
from dateutil.tz import tzoffset

# Gettext needs to be initialized before UC is imported (via profile)
import pytrainer.lib.localization
pytrainer.lib.localization.initialize_gettext("../../locale")

from pytrainer.lib.ddbb import DDBB
from pytrainer.profile import Profile
from pytrainer.lib.uc import UC
from pytrainer.core.activity import ActivityService

class ActivityTest(unittest.TestCase):

    def setUp(self):
        profile = Mock()
        profile.getValue = Mock(return_value='memory')
        self.ddbb = DDBB(profile)
        main = Mock()
        main.ddbb = self.ddbb
        main.profile = Profile()
        main.ddbb.connect()
        main.ddbb.create_tables(add_default=True) # We need a sport
        self.uc = UC()
        self.uc.set_us(False)
        self.service = ActivityService(pytrainer_main=main)
        self.ddbb.insert('records', 'distance,maxspeed,maxpace,title,upositive,average,date_time_local,calories,date_time_utc,comments,pace,unegative,duration,beats,time,date,sport,maxbeats', (46.18, 44.6695617695, 1.2, 'test activity', 553.05993673, 22.3882142185, '2016-07-24 12:58:23+0300', 1462, '2016-07-24T09:58:23Z', 'test comment', 2.4, 564.08076273, 7426, 115.0, '7426', '2016-07-24', 1, 120.0))
        self.ddbb.insert('laps', 'distance,lap_number,calories,avg_hr,elapsed_time,record,intensity,laptrigger,max_hr', (46181.9107740694, 0, 1462, 136, 7426.0, 1, 'active', 'manual', 173))
        self.activity = self.service.get_activity(1)

    def tearDown(self):
        self.service = None
        self.ddbb.disconnect()
        self.ddbb = None
        self.uc.set_us(False)

    def test_activity_date_time(self):
        self.assertEquals(self.activity.date_time, datetime(2016, 7, 24, 12, 58, 23,
                                                            tzinfo=tzoffset(None, 10800)))

    def test_activity_distance(self):
        self.assertEquals(self.activity.distance, 46.18)

    def test_activity_sport_name(self):
        self.assertEquals(self.activity.sport_name, 'Mountain Bike')

    def test_activity_time(self):
        self.assertEquals(self.activity.time, 7426)

    def test_activity_duration(self):
        self.assertEquals(self.activity.time, self.activity.duration)

    def test_activity_starttime(self):
        self.assertEquals(self.activity.starttime, '12:58:23 PM')

    def test_activity_time_tuple(self):
        self.assertEquals(self.activity.time_tuple, (2, 3, 46))

    def test_activity_lap(self):
        self.assertEquals(self.activity.laps[0], {'distance': 46181.9107741, 'end_lon': None, 'lap_number': 0, 'start_lon': None, 'id_lap': 1, 'calories': 1462, 'comments': None, 'laptrigger': u'manual', 'elapsed_time': u'7426.0', 'record': 1, 'intensity': u'active', 'avg_hr': 136, 'max_hr': 173, 'end_lat': None, 'start_lat': None, 'max_speed': None})

    def test_activity_get_value_f(self):
        self.assertEquals(self.activity.get_value_f('distance', "%0.2f"), '46.18')
        self.assertEquals(self.activity.get_value_f('average', "%0.2f"), '22.39')
        self.assertEquals(self.activity.get_value_f('maxspeed', "%0.2f"), '44.67')
        self.assertEquals(self.activity.get_value_f('time', '%s'), '2:03:46')
        self.assertEquals(self.activity.get_value_f('calories', "%0.0f"), '1462')
        self.assertEquals(self.activity.get_value_f('pace', "%s"), '2:24')
        self.assertEquals(self.activity.get_value_f('maxpace', "%s"), '1:12')
        self.assertEquals(self.activity.get_value_f('upositive', "%0.2f"), '553.06')
        self.assertEquals(self.activity.get_value_f('unegative', "%0.2f"), '564.08')

    def test_activity_get_value_f_us(self):
        self.uc.set_us(True)
        self.assertEquals(self.activity.get_value_f('distance', "%0.2f"), '28.69')
        self.assertEquals(self.activity.get_value_f('average', "%0.2f"), '13.91')
        self.assertEquals(self.activity.get_value_f('maxspeed', "%0.2f"), '27.76')
        self.assertEquals(self.activity.get_value_f('time', '%s'), '2:03:46')
        self.assertEquals(self.activity.get_value_f('calories', "%0.0f"), '1462')
        self.assertEquals(self.activity.get_value_f('pace', "%s"), '3:52')
        self.assertEquals(self.activity.get_value_f('maxpace', "%s"), '1:56')
        self.assertEquals(self.activity.get_value_f('upositive', "%0.2f"), '1814.50')
        self.assertEquals(self.activity.get_value_f('unegative', "%0.2f"), '1850.66')

    def test_activity_service_null(self):
        none_activity = self.service.get_activity(None)
        self.assertIsNone(none_activity.id)



#!/usr/bin/env python
from optparse import OptionParser
import os, stat
import sys
import logging
import gtk

import string
from lxml import etree

import httplib, httplib2
import urllib2
import mimetools, mimetypes
from json import dumps, loads       #   for deserializing JSON data form javascript

from pytrainer.extensions.mapviewer import MapViewer
from pytrainer.extensions.osm import Osm

class openstreetmap:
    def __init__(self, parent = None, pytrainer_main = None, conf_dir = None, options = None):
        self.parent = parent
        self.pytrainer_main = pytrainer_main
        self.options = options
        self.conf_dir = conf_dir
        self.description = " "
        self.tags = ""
        self.visibility = "private"
        self.mozTitle=""               # Keeps embedded mozilla document title while displaying map for private area selection
        self.privBounds=[]             # Bounds of areas to be anonymized 

    def run(self, id, activity=None):  #TODO Convert to use activity...
        logging.debug(">>")
        try:
            uri = "http://api.openstreetmap.org/api/0.6/gpx/create" #URI for uploading traces to OSM
            if 'username' not in self.options or self.options['username'] == "" or 'password' not in self.options or self.options['password'] == "":
                logging.error("Must have username and password configured")
                raise Exception("Must have username and password configured")
            username = self.options['username']
            password = self.options['password']
            gpx_file = "%s/gpx/%s.gpx" % (self.conf_dir, id)
            if not os.path.isfile(gpx_file):
                raise Exception(str(gps_file) + ' File not found')
            #GPX file is ok and found, so open it
            logging.debug("GPX file: %s found, size: %d" % (gpx_file, os.path.getsize(gpx_file)))
            f = open(gpx_file, 'r')
            file_contents = f.read()
            #TODO Fix to use etree functionality.....
            if file_contents.find("<?xml version='1.0' encoding='ASCII'?>") != -1:
                logging.debug("GPX file: %s has ASCII encoding - updating to UTF-8 for OSM support" % gpx_file)
                f.close()                   #Close readonly file
                f = open(gpx_file, 'w')     #and open file for writing
                file_contents = file_contents.replace("<?xml version='1.0' encoding='ASCII'?>","<?xml version='1.0' encoding='UTF-8'?>", 1)
                f.write(file_contents)      #Write new content
                f.close()                   #Close
                f = open(gpx_file, 'r')     #Reopen in readonly mode
            #Get extra info from user
            response=self.display_options_window()
            if not response==gtk.RESPONSE_ACCEPT:
                f.close()
                logging.debug("User abort")
                return
            if self.makeanon:
                logging.debug("User requested anonymizing of GPX data")
                f.close()                   #Close standard gpxfile
                gpx_file = self.make_gpx_private(gpx_file)
                f = open(gpx_file, 'r')     #Open anonymous gpxfile in readonly mode
            fields = (("description",self.description), ("tags",self.tags), ("visibility",self.visibility))
            logging.debug("Added fields: %s" % str(fields))
            #Multipart encode the request
            boundary, body = self.multipart_encode(fields=fields, files=(("file", f),))
            content_type = 'multipart/form-data; boundary=%s' % boundary
            #Finished with the file so close it
            f.close()
            #Add the http headers to the request
            h = httplib2.Http()
            headers = {
                'Content-Type': content_type
            }
            #Add basic authentication credentials to the request
            h.add_credentials(username, password)
            #Show user something is happening
            msg = _("Posting GPX trace to Openstreetmap\n\nPlease wait this could take several minutes")
            md = gtk.MessageDialog(self.pytrainer_main.windowmain.window1, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_INFO, gtk.BUTTONS_NONE, msg)
            md.set_title(_("Openstreetmap Extension Processing"))
            md.set_modal(True)
            md.show()
            while gtk.events_pending(): # This allows the GUI to update
                gtk.main_iteration()    # before completion of this entire action
            logging.debug("before request posting")
            #POST request to OSM
            res, content = h.request(uri, 'POST', body=body, headers=headers)
            logging.debug("after request posting")
            logging.debug("Got response status: %s, reason: %s, content: %s" % (res.status, res.reason, content))
            if res.reason == 'OK':
                res_msg = "Successfully posted to OSM.\nYou should get an email with the outcome of the upload soon\n\nTrace id is %s" % content
            else:
                res_msg = "Some error occured\nGot a status %s, reason %s\nContent was: %s" % (res.status, res.reason, content)
            #Close 'Please wait' dialog
            md.destroy()
            #Show the user the result
            md = gtk.MessageDialog(self.pytrainer_main.windowmain.window1, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, res_msg)
            md.set_title(_("Openstreetmap Extension Upload Complete"))
            md.set_modal(False)
            md.run()
            md.destroy()
        except Exception as e:
                msg = _("Error while uploading file to OSM: " + str(e))
                md = gtk.MessageDialog(self.pytrainer_main.windowmain.window1, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, msg)
                md.set_title(_("Openstreetmap Extension Error"))
                md.run()
                md.destroy()
                return
        finally:
            logging.debug("<<")

    def display_options_window(self):
        self.prefwindow = gtk.Dialog(title=_("Please add any additional information for this upload"), parent=None, flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT, gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
        self.prefwindow.set_modal(False)
        table = gtk.Table(1,3)
        self.entryList = []
        #Add description
        label = gtk.Label("<b>Description</b>")
        label.set_use_markup(True)
        entry = gtk.Entry()
        self.entryList.append(entry)
        table.attach(label,0,1,0,1)
        table.attach(entry,1,2,0,1)
        #Add tags
        label = gtk.Label("<b>Tags</b>")
        label.set_use_markup(True)
        entry = gtk.Entry()
        self.entryList.append(entry)
        table.attach(label,0,1,1,2)
        table.attach(entry,1,2,1,2)
        #Add visibility
        label = gtk.Label("<b>Visibility</b>")
        label.set_use_markup(True)
        combobox = gtk.combo_box_new_text()
        combobox.append_text("private")
        combobox.append_text("public")
        combobox.append_text("trackable")
        combobox.append_text("identifiable")
        combobox.set_active(0)
        table.attach(combobox,1,2,2,3)
        self.entryList.append(combobox)
        table.attach(label,0,1,2,3)
        #Add anonymize GPX option
        label = gtk.Label("<b>Anonymize GPX Data</b>")
        label.set_use_markup(True)
        table.attach(label,0,1,3,4)
        checkbutton = gtk.CheckButton()
        table.attach(checkbutton,1,2,3,4)
        self.entryList.append(checkbutton)
        #Add anon area selection button
        button = gtk.Button("Area selection")
        button.connect("clicked",self.areaSelect)
        table.attach(button,1,2,4,5)
        self.entryList.append(button)
        #Build dialog and show
        self.prefwindow.vbox.pack_start(table)
        self.prefwindow.show_all()
        self.prefwindow.connect("response", self.on_options_ok_clicked)
        response=self.prefwindow.run()
        self.prefwindow.destroy()
        return response

    def on_options_ok_clicked(self, widget, response_id):
        if not response_id == gtk.RESPONSE_ACCEPT:
            return response_id
        self.description = self.entryList[0].get_text()
        if self.description == "":
            logging.debug("A description is required - setting to default")
            self.description = "Uploaded from pytrainer"
        self.tags = self.entryList[1].get_text()
        self.visibility = self.entryList[2].get_active_text()
        self.makeanon = self.entryList[3].get_active()
        logging.debug("Description: %s, tags: %s, visibility: %s, makeanon: %s" % ( self.description, self.tags, self.visibility, self.makeanon) )

    def areaSelect(self,dc):
        """
            Open a window with OSM map so user could select his private/anonymized area - 
            all GPX dots in this area will be removed before uploading to OSM
        """       
        try:
            wTree = gtk.glade.XML(self.pytrainer_main.data_path+"extensions/openstreetmap/OSM_AnonSelection.glade")
            self.privAreaWindow = wTree.get_widget("OSM_AnonSelection")
            dic = {
                "on_buttonOk_clicked" : self.privArea_Ok,
                "on_buttonCancel_clicked" : self.privArea_Cancel
            }
            wTree.signal_autoconnect( dic )
            mapviewer = MapViewer(self.pytrainer_main.data_path, pytrainer_main=self.pytrainer_main, box=wTree.get_widget("mapBox"))
            json=None
            if self.options.has_key('privPolygon'):
                json=self.options['privPolygon']
            htmlfile = Osm(data_path=self.pytrainer_main.data_path, waypoint=json, pytrainer_main=self.pytrainer_main).selectArea()
            mapviewer.display_map(htmlfile=htmlfile)
            mapviewer.moz.connect('title', self.parseTitle) 
            self.privAreaWindow.show()

        except Exception as e:
                msg = "Could not init map selection screen, Error: " + str(e)
                md = gtk.MessageDialog(self.pytrainer_main.windowmain.window1, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, msg)
                md.set_title(_("Error"))
                md.run()
                md.destroy()
                return                

    def parseTitle(self, moz):
        "Event fired when document title was changed -> meaning polygon was changed"
        if moz.get_title() != "":
            self.mozTitle=str(moz.get_title())

    def privArea_Cancel(self,button):
        "Event fired when cancel button was pressed"
        self.privAreaWindow.destroy()
        
    def privArea_Ok(self,button):
        "Event fired when ok button was pressed"
        logging.debug(">> privArea_Ok")
        # save new private area polygon if changed
        if self.mozTitle=="":
            return
        # try parsing JSON
        try:
            # verify json is correct by deserializing and serializing it
            polygonString=dumps(loads(self.mozTitle))
            # try saving
            extensionDir = self.pytrainer_main.data_path + "/extensions" + "/openstreetmap"
            if not os.path.isdir(extensionDir):
                loggin.error(str(e))
                print ("Could not find extension path: " + str(extensionDir))
                raise Exception("Could not find extension path: " + str(extensionDir))
            # save new options
            self.options['privPolygon'] = polygonString
            # convert dictionary to a lists set
            savedOptions = []
            for key in self.options:
                savedOptions.append((key,self.options[key]))
            # write new xml config file
            self.parent.setExtensionConfParams(extensionDir, savedOptions)
        except Exception as e:
            logging.error(str(e))    
            print "Error while saving extension configuration: " + str(e)
            msg = _(str(e))
            md = gtk.MessageDialog(self.pytrainer_main.windowmain.window1, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, msg)
            md.set_title(_("Error while saving extension configuration"))
            md.run()
            md.destroy()
        finally:
            self.privAreaWindow.destroy()
                                
    def multipart_encode(self, fields, files, boundary = None, buffer = None):
        '''
            Multipart encode data for posting
             from examples at from http://odin.himinbi.org/MultipartPostHandler.py & http://bitworking.org/projects/httplib2/doc/html/libhttplib2.html
        '''
        if boundary is None:
            boundary = mimetools.choose_boundary()
        if buffer is None:
            buffer = ''
        for (key, value) in fields:
            buffer += '--%s\r\n' % boundary
            buffer += 'Content-Disposition: form-data; name="%s"' % key
            buffer += '\r\n\r\n' + value + '\r\n'
        for (key, fd) in files:
            file_size = os.fstat(fd.fileno())[stat.ST_SIZE]
            filename = os.path.basename(fd.name)
            contenttype = mimetypes.guess_type(filename)[0] or 'application/octet-stream'
            buffer += '--%s\r\n' % boundary
            buffer += 'Content-Disposition: form-data; name="%s"; filename="%s"\r\n' % (key, filename)
            buffer += 'Content-Type: %s\r\n' % contenttype
            # buffer += 'Content-Length: %s\r\n' % file_size
            fd.seek(0)
            buffer += '\r\n' + fd.read() + '\r\n'
        buffer += '--%s--\r\n\r\n' % boundary
        return boundary, buffer
        
    def make_gpx_private(self, gpx_file=None):
        '''
        wipes out private data from gpx files
        converts laps to waypoints
        '''
        logging.debug(">>")
        if gpx_file is None:
            return None
        
        filen = os.path.basename(gpx_file)
        tmpdir = self.pytrainer_main.profile.tmpdir
        anon_gpx_file = "%s/%s" % (tmpdir, filen)

        # get saved private area polygon
        pP=loads(self.options['privPolygon'])
        pP=pP['geometry']['coordinates'][0]
        # converts polygon's 2D matrix into a vector of just the lats or lons
        vector = lambda lonLat: [pP[i][lonLat] for i in range(len(pP))] # 0:lon, 1:lat
        # try reading private area's bounds, stored as [lon,lat]
        NE_LAT = max([pP[i][1] for i in range(len(pP))])
        NE_LON = max([pP[i][0] for i in range(len(pP))])
        SW_LAT = min([pP[i][1] for i in range(len(pP))])
        SW_LON = min([pP[i][0] for i in range(len(pP))])
        logging.info("Anonymizing Area: NE:%f,%f -> SW: %f,%f" % (NE_LON, NE_LAT, SW_LON, SW_LAT))
            
        # Config parameters, not used yet
        FILTER_BOX = True
        ERASE_TIME  = True
        LAP_TO_WAYPOINT = True

        tree = etree.parse(gpx_file)
        _xmlns = tree.getroot().nsmap[None]
        _trkpt_path = '{%s}trk/{%s}trkseg/{%s}trkpt' % (_xmlns, _xmlns, _xmlns)
        # namespace of gpx files
        NS = dict(ns='http://www.topografix.com/GPX/1/1')

        myroot =  tree.getroot()
        gpxdataNS = string.Template(\
            ".//{http://www.cluetrust.com/XML/GPXDATA/1/0}$tag")
        lapTag = gpxdataNS.substitute(tag="lap")
        endPointTag = gpxdataNS.substitute(tag="endPoint")
        triggerTag = gpxdataNS.substitute(tag="trigger")
        laps = tree.findall(lapTag)

        mygpx = tree.find('gpx')

        for lap in laps:
            trigger = lap.find(triggerTag)
            #  Watch out for manually triggered laps
            if trigger.text == 'manual':
                endPoint = lap.find(endPointTag)
                lat = endPoint.get("lat")
                lon = endPoint.get("lon")
                # Create waypt if not in home box
                try:
                    if not ((SW_LAT < float(lat) < NE_LAT) and (SW_LON < float(lon) < NE_LON)):
                        etree.SubElement(myroot, 'wpt', attrib= {'lat':lat, 'lon':lon})
                except:
                    pass
        etree.strip_attributes(myroot, 'creator')
                    
        # Wipe out home box
        for trkpt in tree.findall(_trkpt_path):
            lat = float(trkpt.attrib['lat'])
            lon = float(trkpt.attrib['lon'])
            if (lat < NE_LAT) & (lon < NE_LON) & (lat > SW_LAT) & (lon > SW_LON):
                par = trkpt.getparent()
                par.remove(trkpt)

        time = tree.xpath('//ns:trkpt/ns:time', namespaces=NS)
        for i in time:
            i.text = '1970-01-01T00:00:00+00:00'
            # osm regards <time> as mandatory. gnaa.

        ext = tree.xpath('//ns:gpx/ns:extensions', namespaces=NS)
        for i in ext:
            par = i.getparent()
            par.remove(i)
        meta = tree.xpath('//ns:gpx/ns:metadata', namespaces=NS)
        for i in meta:
            par = i.getparent()
            par.remove(i)
        ele = tree.xpath('//ns:trkpt/ns:ele', namespaces=NS)
        for i in ele:
            par = i.getparent()
            par.remove(i)

        # test schema on cleaned xml-tree
        # gpx.xsd from http://www.topografix.com/gpx.asp

        #xmlschema = etree.XMLSchema(etree.parse('gpx.xsd'))
        #xmlschema.validate(tree)

        # write new gpx file
        tree.write(anon_gpx_file, pretty_print=False, xml_declaration=True, encoding='UTF-8')
        logging.debug("<<")
        return anon_gpx_file




# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import logging

class fileUtils:
    def __init__(self, filename, data):
        self.filename = filename
        self.data = data

    def run(self):
        logging.debug('>>')
        if self.data is not None:
            logging.debug("Writing in %s " % self.filename) 
            out = open(self.filename, 'w')
            out.write(self.data)
            out.close()
        else:
            logging.error("Nothing to write in %s" % self.filename)
        logging.debug('<<')





# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from gui.drawArea import DrawArea
import logging

class HeartRateGraph:
	def __init__(self, vbox = None, window = None, vbox2 = None, pytrainer_main = None):
		logging.debug('>>')
		self.drawarea = DrawArea(vbox, window)
		self.drawarea2 = DrawArea(vbox2, window)
		self.pytrainer_main = pytrainer_main
		logging.debug('<<')

	def drawgraph(self,values):
		logging.debug('>>')
		zones = self.pytrainer_main.profile.getZones()
		xvalues, yvalues = self.get_values(values)
		#logging.debug('xvalues: '+str(xvalues))
		#logging.debug('yvalues: '+str(yvalues))
		xlabel,ylabel,title,color = _("Distance (km)"),_("Beats (bpm)"),_("Heart Rate"),"#740074"
		self.drawarea.stadistics("plot",[xvalues],[yvalues],[xlabel],[ylabel],[title],[color],zones)
		self.drawarea2.stadistics("pie",[xvalues],[yvalues],[xlabel],[ylabel],[title],[color],zones)
		logging.debug('<<')

	def get_values(self,values):
		logging.debug('>>')
		xvalue = []
		yvalue = []
		for value in values:
			xvalue.append(value[0])
			yvalue.append(value[6])
		logging.debug('<<')
		return xvalue,yvalue

	def getFloatValue(self, value):
		try:
			return float(value)
		except:
			return float(0)



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from __future__ import division
from SimpleGladeApp import SimpleBuilderApp
from windowcalendar import WindowCalendar
from pytrainer.core.equipment import EquipmentService
from pytrainer.gui.equipment import EquipmentUi
from pytrainer.core.sport import Sport
import gtk
import gobject
import logging
import pytrainer
import pytrainer.util.color
from pytrainer.gui.color import ColorConverter

class WindowProfile(SimpleBuilderApp):
    def __init__(self, sport_service, data_path = None, parent=None, pytrainer_main=None):
        self.parent = parent
        self.pytrainer_main = pytrainer_main
        self.data_path = data_path
        SimpleBuilderApp.__init__(self, "profile.ui")
        self.conf_options = parent.profile_options
        self.stored_color = pytrainer.util.color.Color(0)
        self._sport_service = sport_service

    def new(self):
        logging.debug(">>")
        self.gender_options = {
            0:_("Male"),
            1:_("Female")
            }

        self.ddbb_type = {
            0:"sqlite",
            1:"mysql"
            }
    
        #anhadimos las opciones al combobox gender
        for i in self.gender_options:
            self.prf_gender.insert_text(i,self.gender_options[i])
        
        #hacemos lo propio para el combobox ddbb 
        for i in self.ddbb_type:
            self.prf_ddbb.insert_text(i,self.ddbb_type[i])

        #preparamos la lista sports:
        column_names=[_("Sport"),_("MET"),_("Extra Weight"), _("Maximum Pace"), _("Color")]
        for column_index, column_name in enumerate(column_names):
            if column_index==4:
                renderer = gtk.CellRendererPixbuf()
            else:
                renderer = gtk.CellRendererText()
            column = gtk.TreeViewColumn(column_name, text=column_index)
            column.pack_start(renderer, expand=False)
            if column_index==4:
                column.add_attribute(renderer, 'pixbuf', column_index)
            else:
                column.add_attribute(renderer, 'text', column_index)
            column.set_resizable(True)
            self.sportTreeView.append_column(column)

        #initialise equipment tab:
        equipment_service = EquipmentService(self.pytrainer_main.ddbb)
        equipment_ui = EquipmentUi(self.data_path + "/glade", equipment_service)
        self.equipment_container.add(equipment_ui) 
        logging.debug("<<")           
        
    def present(self):
        self.newprofile.present()
        
    def setValues(self, list_options):
        logging.debug(">>")
        # Need to think if it does make sense to use pprint -> compatibility issues
        #print list_options
        for i in self.conf_options.keys():
            if not list_options.has_key(i):
                logging.info('No list options for %s' %s)
                continue
            if i == "default_viewer":
                if list_options[i] == "1":
                    logging.info("Setting default map viewer to OSM")
                    self.radiobuttonDefaultOSM.set_active(1)
                else:
                    logging.info("Setting default map viewer to Google")
                    self.radiobuttonDefaultGMap.set_active(1)
            elif i == "prf_startscreen":
                if list_options[i] == "current_day":
                    self.radioButtonStartScreenCurrentDay.set_active(1)
                else:
                    self.radioButtonStartScreenLastEntry.set_active(1)
                logging.info("Setting start screen to display %s" % list_options[i])
            else:
                try:
                    var = getattr(self,i)
                except AttributeError as e:
                    continue                
                if i == "prf_hrzones_karvonen" or i == "prf_us_system":
                    if list_options[i] == "True":
                        logging.info("Setting %s to true" %i)
                        var.set_active(True)
                elif i == "prf_gender":
                    for j in self.gender_options:
                        if self.gender_options[j] == list_options[i]:
                            logging.info("Setting gender to %s" % self.gender_options[j])
                            var.set_active(j)
                elif i == "prf_ddbb":
                    for j in self.ddbb_type:
                        if self.ddbb_type[j] == list_options[i]:
                            logging.info("Setting %s as database engine" % self.ddbb_type[j])
                            var.set_active(j)
                            if j==0:
                                self._ddbb_value_deactive()
                            else:
                                self._ddbb_value_active()
                else:
                    logging.info("Setting %s to %s" % (i,list_options[i]))
                    var.set_text(list_options[i])
        logging.debug("<<")
    
    def saveOptions(self):
        logging.debug(">>")
        list_options = {}
        print self.conf_options
        for i in self.conf_options.keys():
            if i == "default_viewer":
                if self.radiobuttonDefaultOSM.get_active():
                    list_options[i] = "1"
                else:
                    list_options[i] = "0"
            elif i == "prf_startscreen":
                ss_selected = "current_day"
                if self.radioButtonStartScreenLastEntry.get_active():
                    ss_selected = "last_entry"
                list_options[i] = ss_selected
            else:
                try:
                    var = getattr(self,i)
                except AttributeError as e:
                    continue
                if i == "prf_hrzones_karvonen" or i == "prf_us_system":
                    if var.get_active():
                        list_options[i] = "True"
                    else:
                        list_options[i] = "False"
                elif i == "prf_gender" or i == "prf_ddbb":
                    list_options[i] = var.get_active_text()
                else:
                    list_options[i] = var.get_text()
            logging.info("Saving %s as %s" % (i, list_options[i]))
        logging.info("Updating profile...")
        self.parent.setProfile(list_options)
        self.parent.saveProfile()
        logging.debug("<<")
    
    def on_calendar_clicked(self,widget):
        calendardialog = WindowCalendar(self.data_path,self)
        calendardialog.run()

    def setDate(self,date):
        self.prf_age.set_text(date)

    def on_switch_page(self,widget,pointer,frame):
        #print widget, pointer, frame
        if frame==2:
            self.saveOptions()
            sport_list = self._sport_service.get_all_sports()
            store = gtk.ListStore(
                        gobject.TYPE_STRING,
                        gobject.TYPE_STRING,
                        gobject.TYPE_STRING,
                        gobject.TYPE_STRING,
                        gtk.gdk.Pixbuf,
                        object)
            for sport in sport_list:
                iter = store.append()
                colorPixBuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, False, 8, 25, 15)
                colorPixBuf.fill(sport.color.rgba_val)
                store.set(iter,
                    0, sport.name,
                    1, sport.met,
                    2, sport.weight,
                    3, sport.max_pace,
                    4, colorPixBuf)
            self.sportTreeView.set_model(store)
            self.sportTreeView.set_cursor(0)
            self.sportlist.show()
        elif frame == 5: #Startup Parameters page selected
            self.init_params_tab()
    
    def init_params_tab(self):
        #Show log level
        if self.pytrainer_main.startup_options.log_level == logging.ERROR:
            self.comboboxLogLevel.set_active(0)
        elif self.pytrainer_main.startup_options.log_level == logging.WARNING:
            self.comboboxLogLevel.set_active(1)
        elif self.pytrainer_main.startup_options.log_level == logging.INFO:
            self.comboboxLogLevel.set_active(2)
        elif self.pytrainer_main.startup_options.log_level == logging.DEBUG:
            self.comboboxLogLevel.set_active(3)
        else:
            self.comboboxLogLevel.set_active(0)
            print "Unknown logging level specified"

        #Show if validation requested
        if self.pytrainer_main.startup_options.validate:
            self.checkbuttonValidate.set_active(True)
        else:
            self.checkbuttonValidate.set_active(False)
            
        #Show if new graph activated
        if self.pytrainer_main.startup_options.newgraph:
            self.checkbuttonNewGraph.set_active(True)
        else:
            self.checkbuttonNewGraph.set_active(False)
        
    def on_comboboxLogLevel_changed(self, widget):
        active = self.comboboxLogLevel.get_active()
        if active == 1:
            logging.debug("Setting log level to WARNING")
            self.pytrainer_main.startup_options.log_level = logging.WARNING
        elif active == 2:
            logging.debug("Setting log level to INFO")
            self.pytrainer_main.startup_options.log_level = logging.INFO
        elif active == 3:
            logging.debug("Setting log level to DEBUG")
            self.pytrainer_main.startup_options.log_level = logging.DEBUG
        else:
            logging.debug("Setting log level to ERROR")
            self.pytrainer_main.startup_options.log_level = logging.ERROR
        self.pytrainer_main.set_logging_level(self.pytrainer_main.startup_options.log_level)    
        
    def on_checkbuttonValidate_toggled(self, widget):
        if self.checkbuttonValidate.get_active():
            logging.debug( "Validate activated")
            self.pytrainer_main.startup_options.validate = True
        else:
            logging.debug("Validate deactivated")
            self.pytrainer_main.startup_options.validate = False
            
    def on_checkbuttonNewGraph_toggled(self, widget):
        if self.checkbuttonNewGraph.get_active():
            logging.debug("NewGraph activated")
            self.pytrainer_main.startup_options.newgraph = True
        else:
            logging.debug("NewGraph deactivated")
            self.pytrainer_main.startup_options.newgraph = False       
    
    def on_accept_clicked(self,widget):
        self.saveOptions()
        self.close_window()
    
    def on_cancel_clicked(self,widget):
        self.close_window()

    def close_window(self):
        self.newprofile.hide()
        self.newprofile = None
        self.quit()

    def _ddbb_value_active(self):
        self.prf_ddbbhost_label.set_sensitive(1)
        self.prf_ddbbname_label.set_sensitive(1)
        self.prf_ddbbuser_label.set_sensitive(1)
        self.prf_ddbbpass_label.set_sensitive(1)
        self.prf_ddbbhost.set_sensitive(1)
        self.prf_ddbbname.set_sensitive(1)
        self.prf_ddbbuser.set_sensitive(1)
        self.prf_ddbbpass.set_sensitive(1)
    
    def _ddbb_value_deactive(self):
        self.prf_ddbbhost_label.set_sensitive(0)
        self.prf_ddbbname_label.set_sensitive(0)
        self.prf_ddbbuser_label.set_sensitive(0)
        self.prf_ddbbpass_label.set_sensitive(0)
        self.prf_ddbbhost.set_sensitive(0)
        self.prf_ddbbname.set_sensitive(0)
        self.prf_ddbbuser.set_sensitive(0)
        self.prf_ddbbpass.set_sensitive(0)

    def on_prf_ddbb_changed(self,widget):
        i = self.prf_ddbb.get_active_text()
        if i == "mysql":
            self._ddbb_value_active()
        else:
            self._ddbb_value_deactive()

    def on_addsport_clicked(self,widget):
        self.hidesportsteps()
        self.buttonbox.set_sensitive(0)
        self.addsport.show()

    def on_newsport_accept_clicked(self,widget):
        sport = Sport()
        sport.name = unicode(self.newsportentry.get_text())
        sport.met = self._trim_to_null(self.newmetentry.get_text())
        sport.weight = self.newweightentry.get_text()
        sport.max_pace = self._trim_to_null(self.newmaxpace.get_text())
        sport.color = self.stored_color
        if sport.name.lower() in [s.name.lower() for s in self._sport_service.get_all_sports()]:
            msg = "Sport '%s' already exists" % sport.name
            logging.error(msg)
            md = gtk.MessageDialog(None, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, _(msg))
            md.set_title(_("Sport Creation Error"))
            md.run()
            md.destroy()
            return
        self._sport_service.store_sport(sport)
        self.pytrainer_main.refreshMainSportList()
        self.on_switch_page(None,None,2)
        self.hidesportsteps()
        self.buttonbox.set_sensitive(1)
        self.sportlist.show()

    def on_delsport_clicked(self,widget):
        selected,iter = self.sportTreeView.get_selection().get_selected()
        if iter:
            self.buttonbox.set_sensitive(0)
            sport = selected.get_value(iter,0)
            self.sportnamedel.set_text(sport)
            self.hidesportsteps()
            self.deletesport.show()

    def on_deletesport_clicked(self,widget):
        sport_name = self.sportnamedel.get_text()
        sport = self._sport_service.get_sport_by_name(sport_name)
        self._sport_service.remove_sport(sport)
        self.pytrainer_main.refreshMainSportList()
        self.on_switch_page(None,None,2)
        self.hidesportsteps()
        self.buttonbox.set_sensitive(1)
        self.sportlist.show()
    
    def on_editsport_clicked(self,widget):
        selected,iter = self.sportTreeView.get_selection().get_selected()
        if iter:
            self.buttonbox.set_sensitive(0)
            sport_desc = selected.get_value(iter,0)
            sport = self._sport_service.get_sport_by_name(sport_desc)
            self.editsportentry.set_text(sport.name)
            self.sportnameedit.set_text(sport.name)
            self.editweightentry.set_text(str(sport.weight))
            met_str = "" if sport.met is None else str(sport.met)
            self.editmetentry.set_text(met_str)
            max_pace_str = "" if sport.max_pace is None else str(sport.max_pace)
            self.editmaxpace.set_text(max_pace_str)
            colorPixBuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, False, 8, 250, 20)
            colorPixBuf.fill(sport.color.rgba_val)
            self.editcolor.set_from_pixbuf(colorPixBuf)
            self.hidesportsteps()
            self.editsport.show()
            
    def on_editcolor_clicked(self, widget):
        selected,iter = self.sportTreeView.get_selection().get_selected()
        if iter:
            sport_desc = selected.get_value(iter,0)
            sport = self._sport_service.get_sport_by_name(sport_desc)
            colorseldlg = gtk.ColorSelectionDialog("test")
            colorseldlg.colorsel.set_has_palette(True)
            colorseldlg.colorsel.set_current_color(ColorConverter().convert_to_gdk_color(sport.color))
            colorseldlg.run()
            gdk_color = colorseldlg.colorsel.get_current_color()
            self.stored_color = ColorConverter().convert_to_color(gdk_color)
            colorPixBuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, False, 8, 250, 20)
            colorPixBuf.fill(self.stored_color.rgba_val)
            self.newcolor.set_from_pixbuf(colorPixBuf)
            self.editcolor.set_from_pixbuf(colorPixBuf)
            
            colorseldlg.hide()
            
    def on_sporttreeview_row_activated(self, widget, path, column):
        self.on_editsport_clicked(None)
    
    def on_editsport_accept_clicked(self,widget):
        oldnamesport = self.sportnameedit.get_text()
        sport = self._sport_service.get_sport_by_name(oldnamesport)
        sport.name = unicode(self.editsportentry.get_text())
        sport.weight = self.editweightentry.get_text()
        sport.met = self._trim_to_null(self.editmetentry.get_text())
        sport.max_pace = self._trim_to_null(self.editmaxpace.get_text())
        sport.color = self.stored_color
        self._sport_service.store_sport(sport)
        self.pytrainer_main.refreshMainSportList()
        self.on_switch_page(None,None,2)
        self.hidesportsteps()
        self.buttonbox.set_sensitive(1)
        self.sportlist.show()
        
    def _trim_to_null(self, string):
        trimmed = string.strip()
        return None if trimmed == "" else trimmed
        
    def on_sportcancel_clicked(self,widget):
        self.hidesportsteps()
        self.buttonbox.set_sensitive(1)
        self.sportlist.show()

    def on_calculatemaxhr_clicked(self,widget=None):
        import datetime
        today = "%s"%datetime.date.today()
        year1,month1,day1 = today.split("-")
        year2,month2,day2 = self.prf_age.get_text().split("-")
        diff = datetime.datetime(int(year1), int(month1), int(day1),0,0,0) - datetime.datetime(int(year2), int(month2), int(day2),0,0,0)
        self.prf_maxhr.set_text("%d" %(220-int(diff.days/365)))

    def hidesportsteps(self):
        self.sportlist.hide()
        self.addsport.hide()
        self.deletesport.hide()
        self.editsport.hide()   



# -*- coding: iso-8859-1 -*-

#Copyright (C) Sigurður H. Pálsson shp@itn.is

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import datetime
import logging
from gui.drawArea import DrawArea

class TimeGraph(object):
    def __init__(self, sports, vbox = None, window = None, combovalue = None, combovalue2 = None, main = None):
        self.drawarea = DrawArea(vbox, window)
        self.SPORT_FIELD = 9
        self.sport_colors = dict([(sport.name, sport.color.to_hex_string()) for sport in sports])

    def getFloatValue(self, value):
        try:
            return float(value)
        except:
            return float(0)

    def getValue(self,record,value_selected):
        #hacemos una relacion entre el value_selected y los values / we make a relation between value_selected and the values
        conv = {
            0: 1, #value 0 es kilometros (1)
            1: 2, #value 1 es tiempo (2)
            2: 3, #value 2 es pulsaciones(3)
            3: 5, #value 3 es media(5)
            4: 6 #value 4 es calorias(6)
            }
        value_sel = conv[value_selected]
        #si la opcion es tiempo lo pasamos a horas / if the option is time we passed it to hours
        if (value_sel == 2):
            return self.getFloatValue(record[value_sel])/3600
        else:
            return self.getFloatValue(record[value_sel])
    
    def get_values(self, values, value_selected, key_format, sportfield=9):
        valueDict = {} #Stores the totals
        valueCount = {} #Counts the totals to allow for averaging if needed
        sportColors = {}

        for record in values:
            if record[0]:
                day = unicode(datetime.datetime.strptime(record[0], "%Y-%m-%d").strftime(key_format)) # Gives year for this record
                sport = record[sportfield]
                value = self.getValue(record, value_selected)
                if sport in valueDict: #Already got this sport
                    if day in valueDict[sport]: #Already got this sport on this day
                        valueDict[sport][day] += value
                        valueCount[sport][day] += 1
                    else: #New day for this sport
                        valueDict[sport][day] = value
                        valueCount[sport][day] = 1
                else: #New sport
                    valueDict[sport] = {day: value}
                    valueCount[sport] = {day: 1}
            else:
                logging.debug("No date string found, skipping entry: " + str(record))
                
        if value_selected in (2, 3):
            total = {}
            count = {}
            for sport in valueDict.keys():
                for day in valueDict[sport].keys():
                    if valueCount[sport][day] > 1: #Only average if 2 or more entries on this day
                        valueDict[sport][day] /= valueCount[sport][day]

        if value_selected == 1: #Values are of time type
            valuesAreTime=True
        else:
            valuesAreTime=False

        return valueDict, valuesAreTime

    def drawgraph(self,values, extra=None, x_func=None):
        xval = []
        yval = []
        xlab = []
        ylab = []
        tit = []
        
        valsAreTime = []
        value_selected = self.combovalue.get_active()
        value_selected2 = self.combovalue2.get_active()
        if value_selected < 0:
            self.combovalue.set_active(0)
            value_selected = 0

        y1,ylabel,title,y2 = self.get_value_params(value_selected)
        ylab.append(ylabel)
        tit.append(title)

        yvalues, valuesAreTime = self.get_values(values,value_selected, self.KEY_FORMAT, sportfield=self.SPORT_FIELD)
        if not len(values): return
        
        xvalues = x_func(yvalues) 
        
        yval.append(yvalues)
        xlab.append(xvalues)
        valsAreTime.append(valuesAreTime)

        #Second combobox used
        if value_selected2 > 0:
            value_selected2 = value_selected2-1
            y1, ylabel,title,y2 = self.get_value_params(value_selected2)
            ylab.append(ylabel)
            tit.append(title)
            yvalues, valuesAreTime = self.get_values(values,value_selected2, self.KEY_FORMAT, sportfield=self.SPORT_FIELD)
            yval.append(yvalues)
            xlab.append(xvalues)
            valsAreTime.append(valuesAreTime)
        #Draw chart
        self.drawarea.drawStackedBars(xlab,yval,ylab,tit,valsAreTime, colors = self.sport_colors)

    def get_value_params(self,value):
        return self.value_params[value]




# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import logging
from pytrainer.lib.date import unixtime2date

class Waypoint:
	def __init__(self, data_path = None, parent = None):
		logging.debug(">>")
		self.parent = parent
		self.pytrainer_main = parent
		self.data_path = data_path
		logging.debug("<<")
	
	def removeWaypoint(self,id_waypoint):
		logging.debug(">>")
		#self.pytrainer_main.ddbb.connect()
		logging.debug("Deleting id_waypoint=%s" %id_waypoint)
		self.pytrainer_main.ddbb.delete("waypoints", "id_waypoint=\"%s\"" %id_waypoint)
		#self.pytrainer_main.ddbb.disconnect()
		logging.debug("<<")

	def updateWaypoint(self,id_waypoint,lat,lon,name,desc,sym):
		logging.debug(">>")
		#self.pytrainer_main.ddbb.connect()
		logging.debug("Updating waypoint id: %d with lat %s,lon %s,comment %s,name %s,sym %s" %(id_waypoint,lat,lon,desc,name,sym) )
		self.pytrainer_main.ddbb.update("waypoints","lat,lon,comment,name,sym",[lat,lon,desc,name,sym]," id_waypoint=%d" %id_waypoint)
		#self.pytrainer_main.ddbb.disconnect()
		logging.debug("<<")
		
	def addWaypoint(self,lon=None,lat=None,name=None,comment=None,sym=None): 
		logging.debug(">>") 
		#self.pytrainer_main.ddbb.connect() 
		cells = "lat,lon,comment,name,sym" 
		values = (lat,lon,comment,name,sym) 
		logging.debug("Adding waypoint with details lat %s,lon %s,comment %s,name %s,sym %s" % (lat,lon,comment,name,sym)  )
		self.pytrainer_main.ddbb.insert("waypoints",cells,values) 
		id_waypoint = self.pytrainer_main.ddbb.lastRecord("waypoints") 
		#self.pytrainer_main.ddbb.disconnect() 
		logging.debug("<<") 
		return id_waypoint

	def getwaypointInfo(self,id_waypoint):
		logging.debug(">>")
		#self.pytrainer_main.ddbb.connect()
		retorno = self.pytrainer_main.ddbb.select("waypoints",
					"lat,lon,ele,comment,time,name,sym",
					"id_waypoint=\"%s\"" %id_waypoint)
		#self.pytrainer_main.ddbb.disconnect()
		logging.debug("<<")
		return retorno
	
	def getAllWaypoints(self):
		logging.debug(">>")
		#self.pytrainer_main.ddbb.connect()
		retorno = self.pytrainer_main.ddbb.select("waypoints","id_waypoint,lat,lon,ele,comment,time,name,sym","1=1 order by name")
		#self.pytrainer_main.ddbb.disconnect()
		logging.debug("<<")
		return retorno
	
	def actualize_fromgpx(self,gpxfile):
		logging.debug(">>")
		#self.pytrainer_main.ddbb.connect()
		from lib.gpx import Gpx
		gpx = Gpx(self.data_path,gpxfile)
		tracks = gpx.getTrackRoutes()

		if len(tracks) > 1:
			time = unixtime2date(tracks[0][1])
			self.recordwindow.rcd_date.set_text(time)
			self._actualize_fromgpx(gpx)
		else:
			msg = _("The gpx file seems to be a several days records. Perhaps you will need to edit your gpx file")
			from gui.warning import Warning
			warning = Warning(self.data_path,self._actualize_fromgpx,[gpx])
                        warning.set_text(msg)
                        warning.run()
		#self.pytrainer_main.ddbb.disconnect()
		logging.debug("<<")

	def _actualize_fromgpx(self, gpx):
		logging.debug(">>")
		distance, time = gpx.getMaxValues()
		upositive,unegative = gpx.getUnevenness()
		self.recordwindow.rcd_upositive.set_text(str(upositive))
		self.recordwindow.rcd_unegative.set_text(str(unegative))
		self.recordwindow.set_distance(distance)
		self.recordwindow.set_recordtime(time/60.0/60.0)
		self.recordwindow.on_calcavs_clicked(None)
		logging.debug("<<")



# -*- coding: iso-8859-1 -*-

#Copyright (C) Fiz Vazquez vud1@sindominio.net
# vud1@grupoikusnet.com
# Jakinbidea & Grupo Ikusnet Developer
# Modified by dgranda

#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import os
import string
import logging
import xml.dom
from xml.dom import minidom, Node, getDOMImplementation
import xml.etree.cElementTree

# use of namespaces is mandatory if defined
mainNSGarmin = string.Template("{http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v1}$tag")
timeTag = mainNSGarmin.substitute(tag="time")
trackTag = mainNSGarmin.substitute(tag="trk")
trackPointTag = mainNSGarmin.substitute(tag="trkpt")

class XMLParser:
	def __init__(self,filename = None):
		self.filename = filename
		self._load()

	def _load(self):
		logging.debug("Loading %s" % self.filename)
		try: 
			self.xmldoc = xml.dom.minidom.parse(self.filename) 
		except:
			self.xmldoc = None
			
	def setOptions(self,list_item, version):
		list_item.append(("version",version))
		self.createXMLFile(self,"pytraining",list_item)

	def getOptions(self):
		#self._load()
		root = self.xmldoc.getElementsByTagName("pytraining")[0]
		list_options = {}
		list_keys = root.attributes.keys()
		for i in list_keys:
			value = self.getValue("pytraining",i)
			list_options[i] = value
		return list_options	
	
	def getOption(self,option):
		logging.info("this function is obsolete, use getValue instead")
		return self.getValue("pytraining",option)
		
	def setVersion(self,version):
		self.setValue("pytraining","version",version)

	def setValue(self,tagname,variable,value):
		root = self.xmldoc.getElementsByTagName(tagname)[0]
		if root.attributes.has_key(variable):
			root.attributes[variable]._set_value(value)
		else:
			root.setAttribute(variable,value)
		content = self.xmldoc.toprettyxml()
		self._saveFile(content)

	def getValue(self,tagname,variable):
		#self._load()
		try:
			root = self.xmldoc.getElementsByTagName(tagname)[0]
			value = root.attributes[variable].value
		except:
			value = None
		return value
	
	def getAllValues(self,tagname):	
		#self._load()
		root = self.xmldoc.getElementsByTagName(tagname)
		retorno = []
		for i in root:
			retorno.append((i.attributes["variable"].value, i.attributes["value"].value))
		return retorno
	
	def createXMLFile(self,tag,list_options):
		impl = getDOMImplementation()
		self.xmldoc = impl.createDocument(None, tag, None)
		top_element = self.xmldoc.documentElement
		#top_element.appendChild(text)
		for option in list_options:
			var = option[0]
			value = option[1]
			attr = self.xmldoc.createAttribute(var)
			top_element.setAttributeNode(attr)
			top_element.attributes[var]._set_value(value)
		xmlcontent = self.xmldoc.toprettyxml()
		self._saveFile(xmlcontent)	

	def _saveFile(self,xmlcontent):
		out = open(self.filename, 'w')
		out.write(xmlcontent)
		out.close()
